from abc import (
    ABCMeta,
    abstractmethod,
)

from ._Sleeper import Sleeper


class AwakableSleeper(Sleeper):
    __meta__ = ABCMeta

    def hibernate(self, secs):
        # type: (float) -> None
        raise NotImplementedError()

    @abstractmethod
    def wake_up(self):
        # type: () -> None
        raise NotImplementedError()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""barcode.base

"""

from barcode.writer import SVGWriter


class Barcode(object):

    name = ''

    raw = None

    digits = 0

    default_writer = SVGWriter

    default_writer_options = {
        'module_width': 0.2,
        'module_height': 15.0,
        'quiet_zone': 6.5,
        'font_size': 10,
        'text_distance': 5.0,
        'background': 'white',
        'foreground': 'black',
        'write_text': True,
        'text': '',
    }

    def to_ascii(self):
        code = self.build()
        for i, line in enumerate(code):
            code[i] = line.replace('1', 'X').replace('0', ' ')
        return '\n'.join(code)

    def __repr__(self):
        return '<{0}({1!r})>'.format(self.__class__.__name__,
                                     self.get_fullcode())

    def build(self):
        raise NotImplementedError

    def get_fullcode(self):
        """Returns the full code, encoded in the barcode.

        :returns: Full human readable code.
        :rtype: String
        """
        raise NotImplementedError

    def save(self, filename, options=None, text=None):
        """Renders the barcode and saves it in `filename`.

        :parameters:
            filename : String
                Filename to save the barcode in (without filename
                extension).
            options : Dict
                The same as in `self.render`.
            text : str (unicode on Python 2)
                Text to render under the barcode.

        :returns: The full filename with extension.
        :rtype: String
        """
        if text:
            output = self.render(options, text)
        else:
            output = self.render(options)

        _filename = self.writer.save(filename, output)
        return _filename

    def write(self, fp, options=None, text=None):
        """Renders the barcode and writes it to the file like object
        `fp`.

        :parameters:
            fp : File like object
                Object to write the raw data in.
            options : Dict
                The same as in `self.render`.
            text : str (unicode on Python 2)
                Text to render under the barcode.
        """
        output = self.render(options, text)
        if hasattr(output, 'tostring'):
            output.save(fp, format=self.writer.format)
        else:
            fp.write(output)

    def render(self, writer_options=None, text=None):
        """Renders the barcode using `self.writer`.

        :parameters:
            writer_options : Dict
                Options for `self.writer`, see writer docs for details.
            text : str (unicode on Python 2)
                Text to render under the barcode.

        :returns: Output of the writers render method.
        """
        options = Barcode.default_writer_options.copy()
        options.update(writer_options or {})
        if options['write_text'] or text is not None:
            if text is not None:
                options['text'] = text
            else:
                options['text'] = self.get_fullcode()
        self.writer.set_options(options)
        code = self.build()
        raw = Barcode.raw = self.writer.render(code)
        return raw



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import s3transfer


KB = 1024
MB = KB * KB
GB = MB * KB

ALLOWED_DOWNLOAD_ARGS = [
    'VersionId',
    'SSECustomerAlgorithm',
    'SSECustomerKey',
    'SSECustomerKeyMD5',
    'RequestPayer',
    'ExpectedBucketOwner',
]

USER_AGENT = 's3transfer/%s' % s3transfer.__version__
PROCESS_USER_AGENT = '%s processpool' % USER_AGENT



from typing import (
    Any,
    Dict,
    Optional,
)


class UnlockVoucherRequest(object):
    def __init__(
        self,                   # type: UnlockVoucherRequest
        benefit_id,             # type: str
        benefit,                # type: Dict[str, Any]
        order_picture,          # type: str
        table_picture,          # type: Optional[str]
    ):
        # type: (...) -> None

        self.benefit_id = benefit_id
        self.benefit = benefit
        self.order_picture = order_picture
        self.table_picture = table_picture



# brn.py - functions for handling South Korean BRN
# coding: utf-8
#
# Copyright (C) 2020 Leandro Regueiro
# Copyright (C) 2020 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""BRN (사업자 등록 번호, South Korea Business Registration Number).

The Business Registration Number is issued by the district tax office in the
local jurisdiction for tax purposes. The number consists of 10 digits and
contain the tax office number (3 digits), the type of business (2 digits), a
serially assigned value (4 digits) and a single check digit.

More information:

* https://www.oecd.org/tax/automatic-exchange/crs-implementation-and-assistance/tax-identification-numbers/Korea-TIN.pdf

>>> validate('116-82-00276')
'1168200276'
>>> validate('1168200276')
'1168200276'
>>> validate(' 116 - 82 - 00276  ')
'1168200276'
>>> validate('123456789')
Traceback (most recent call last):
    ...
InvalidLength: ...
>>> format('1348672683')
'134-86-72683'
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation.

    This strips the number of any valid separators and removes surrounding
    whitespace.
    """
    return clean(number, ' -').strip()


def validate(number):
    """Check if the number is a valid South Korea BRN number.

    This checks the length and formatting.
    """
    number = compact(number)
    if len(number) != 10:
        raise InvalidLength()
    if not isdigits(number):
        raise InvalidFormat()
    if number[:3] < '101' or number[3:5] == '00' or number[5:-1] == '0000':
        raise InvalidComponent()
    return number


def is_valid(number):
    """Check if the number is a valid South Korea BRN number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    number = compact(number)
    return '-'.join([number[:3], number[3:5], number[5:]])



from xml.etree import cElementTree as eTree

from common.builder_functions import optional
from helper import round_half_away_from_zero


class FiscalOrderTotals:

    def __init__(
        self,
        fiscal_xml: eTree.Element,
        total_value_key: str = ".//ICMSTot/vProd",
        total_discount_key: str = ".//ICMSTot/vDesc",
        amount_change_key: str = ".//vTroco",
        total_other_values_key: str = ".//ICMSTot/vOutro",
        total_delivery_key: str = ".//ICMSTot/vFrete",
    ) -> None:

        self.total_value = float(fiscal_xml.find(total_value_key).text)
        self.total_discount = float(fiscal_xml.find(total_discount_key).text or 0)
        self.amount_change = float(optional(element=fiscal_xml.find(amount_change_key)) or 0)
        self.total_paid = round_half_away_from_zero(
            number=self.total_value - self.total_discount,
            precision=2,
        )
        self.total_other_values = float(optional(element=fiscal_xml.find(total_other_values_key)) or 0)
        self.total_delivery = float(optional(element=fiscal_xml.find(total_delivery_key)) or 0)
        self.total_addition = round_half_away_from_zero(
            number=self.total_other_values + self.total_delivery,
            precision=2,
        )



from typing import (
    List,
    Optional,
    Union,
)

from mw_helper import ensure_iterable

from production.model import ProductionOrder
from ._OrderChangerProductionBox import OrderChangerProductionBox


class CustomPropertyValueFilterBox(OrderChangerProductionBox):
    def __init__(
        self,                       # type: CustomPropertyValueFilterBox
        name,                       # type: str
        sons,                       # type: Union[str, List[str]]
        parameter_name,             # type: str
        allowed_values=None,        # type: Optional[List[str]]
        excluded_values=None,       # type: Optional[List[str]]
        remove_order=True,          # type: Optional[bool]
        logger=None
    ):
        # type (...) -> None

        super(CustomPropertyValueFilterBox, self).__init__(name, sons, logger)

        self.parameter_name = parameter_name
        self.allowed_values = [x.lower() for x in ensure_iterable(allowed_values or [])]
        self.excluded_values = [x.lower() for x in ensure_iterable(excluded_values or [])]
        self.remove_order = remove_order

    def change_order(
        self,       # type: CustomPropertyValueFilterBox
        order,      # type: ProductionOrder
    ):
        # type (...) -> ProductionOrder

        if self.should_filter_order(order):
            if self.remove_order:
                return None

            order.items = []

        return order

    def should_filter_order(
        self,       # type: CustomPropertyValueFilterBox
        order,      # type: ProductionOrder
    ):
        # type: (ProductionOrder) -> bool

        if self.parameter_name in order.properties:
            prop_value = order.properties[self.parameter_name].lower()
            if self.allowed_values and prop_value not in self.allowed_values:
                return True

            if self.excluded_values and prop_value in self.excluded_values:
                return True

        return False



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import absolute_import, division, print_function

import six

from cryptography.utils import int_from_bytes, int_to_bytes


# This module contains a lightweight DER encoder and decoder. See X.690 for the
# specification. This module intentionally does not implement the more complex
# BER encoding, only DER.
#
# Note this implementation treats an element's constructed bit as part of the
# tag. This is fine for DER, where the bit is always computable from the type.


CONSTRUCTED = 0x20
CONTEXT_SPECIFIC = 0x80

INTEGER = 0x02
BIT_STRING = 0x03
OCTET_STRING = 0x04
NULL = 0x05
OBJECT_IDENTIFIER = 0x06
SEQUENCE = 0x10 | CONSTRUCTED
SET = 0x11 | CONSTRUCTED
PRINTABLE_STRING = 0x13
UTC_TIME = 0x17
GENERALIZED_TIME = 0x18


class DERReader(object):
    def __init__(self, data):
        self.data = memoryview(data)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        if exc_value is None:
            self.check_empty()

    def is_empty(self):
        return len(self.data) == 0

    def check_empty(self):
        if not self.is_empty():
            raise ValueError("Invalid DER input: trailing data")

    def read_byte(self):
        if len(self.data) < 1:
            raise ValueError("Invalid DER input: insufficient data")
        ret = six.indexbytes(self.data, 0)
        self.data = self.data[1:]
        return ret

    def read_bytes(self, n):
        if len(self.data) < n:
            raise ValueError("Invalid DER input: insufficient data")
        ret = self.data[:n]
        self.data = self.data[n:]
        return ret

    def read_any_element(self):
        tag = self.read_byte()
        # Tag numbers 31 or higher are stored in multiple bytes. No supported
        # ASN.1 types use such tags, so reject these.
        if tag & 0x1f == 0x1f:
            raise ValueError("Invalid DER input: unexpected high tag number")
        length_byte = self.read_byte()
        if length_byte & 0x80 == 0:
            # If the high bit is clear, the first length byte is the length.
            length = length_byte
        else:
            # If the high bit is set, the first length byte encodes the length
            # of the length.
            length_byte &= 0x7f
            if length_byte == 0:
                raise ValueError(
                    "Invalid DER input: indefinite length form is not allowed "
                    "in DER"
                )
            length = 0
            for i in range(length_byte):
                length <<= 8
                length |= self.read_byte()
                if length == 0:
                    raise ValueError(
                        "Invalid DER input: length was not minimally-encoded"
                    )
            if length < 0x80:
                # If the length could have been encoded in short form, it must
                # not use long form.
                raise ValueError(
                    "Invalid DER input: length was not minimally-encoded"
                )
        body = self.read_bytes(length)
        return tag, DERReader(body)

    def read_element(self, expected_tag):
        tag, body = self.read_any_element()
        if tag != expected_tag:
            raise ValueError("Invalid DER input: unexpected tag")
        return body

    def read_single_element(self, expected_tag):
        with self:
            return self.read_element(expected_tag)

    def read_optional_element(self, expected_tag):
        if len(self.data) > 0 and six.indexbytes(self.data, 0) == expected_tag:
            return self.read_element(expected_tag)
        return None

    def as_integer(self):
        if len(self.data) == 0:
            raise ValueError("Invalid DER input: empty integer contents")
        first = six.indexbytes(self.data, 0)
        if first & 0x80 == 0x80:
            raise ValueError("Negative DER integers are not supported")
        # The first 9 bits must not all be zero or all be ones. Otherwise, the
        # encoding should have been one byte shorter.
        if len(self.data) > 1:
            second = six.indexbytes(self.data, 1)
            if first == 0 and second & 0x80 == 0:
                raise ValueError(
                    "Invalid DER input: integer not minimally-encoded"
                )
        return int_from_bytes(self.data, "big")


def encode_der_integer(x):
    if not isinstance(x, six.integer_types):
        raise ValueError("Value must be an integer")
    if x < 0:
        raise ValueError("Negative integers are not supported")
    n = x.bit_length() // 8 + 1
    return int_to_bytes(x, n)


def encode_der(tag, *children):
    length = 0
    for child in children:
        length += len(child)
    chunks = [six.int2byte(tag)]
    if length < 0x80:
        chunks.append(six.int2byte(length))
    else:
        length_bytes = int_to_bytes(length)
        chunks.append(six.int2byte(0x80 | len(length_bytes)))
        chunks.append(length_bytes)
    chunks.extend(children)
    return b"".join(chunks)



# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
# <see AUTHORS file>
#
# This module is part of SQLAlchemy and is released under
# the MIT License: https://www.opensource.org/licenses/mit-license.php
# mypy: ignore-errors

r"""
.. dialect:: oracle+oracledb
    :name: python-oracledb
    :dbapi: oracledb
    :connectstring: oracle+oracledb://user:pass@hostname:port[/dbname][?service_name=<service>[&key=value&key=value...]]
    :url: https://oracle.github.io/python-oracledb/

python-oracledb is released by Oracle to supersede the cx_Oracle driver.
It is fully compatible with cx_Oracle and features both a "thin" client
mode that requires no dependencies, as well as a "thick" mode that uses
the Oracle Client Interface in the same way as cx_Oracle.

.. seealso::

    :ref:`cx_oracle` - all of cx_Oracle's notes apply to the oracledb driver
    as well.

Thick mode support
------------------

By default the ``python-oracledb`` is started in thin mode, that does not
require oracle client libraries to be installed in the system. The
``python-oracledb`` driver also support a "thick" mode, that behaves
similarly to ``cx_oracle`` and requires that Oracle Client Interface (OCI)
is installed.

To enable this mode, the user may call ``oracledb.init_oracle_client``
manually, or by passing the parameter ``thick_mode=True`` to
:func:`_sa.create_engine`. To pass custom arguments to ``init_oracle_client``,
like the ``lib_dir`` path, a dict may be passed to this parameter, as in::

    engine = sa.create_engine("oracle+oracledb://...", thick_mode={
        "lib_dir": "/path/to/oracle/client/lib", "driver_name": "my-app"
    })

.. seealso::

    https://python-oracledb.readthedocs.io/en/latest/api_manual/module.html#oracledb.init_oracle_client


.. versionadded:: 2.0.0 added support for oracledb driver.

"""  # noqa
import re

from .cx_oracle import OracleDialect_cx_oracle as _OracleDialect_cx_oracle
from ... import exc


class OracleDialect_oracledb(_OracleDialect_cx_oracle):
    supports_statement_cache = True
    driver = "oracledb"

    def __init__(
        self,
        auto_convert_lobs=True,
        coerce_to_decimal=True,
        arraysize=50,
        encoding_errors=None,
        thick_mode=None,
        **kwargs,
    ):
        super().__init__(
            auto_convert_lobs,
            coerce_to_decimal,
            arraysize,
            encoding_errors,
            **kwargs,
        )

        if self.dbapi is not None and (
            thick_mode or isinstance(thick_mode, dict)
        ):
            kw = thick_mode if isinstance(thick_mode, dict) else {}
            self.dbapi.init_oracle_client(**kw)

    @classmethod
    def import_dbapi(cls):
        import oracledb

        return oracledb

    @classmethod
    def is_thin_mode(cls, connection):
        return connection.connection.dbapi_connection.thin

    def _load_version(self, dbapi_module):
        version = (0, 0, 0)
        if dbapi_module is not None:
            m = re.match(r"(\d+)\.(\d+)(?:\.(\d+))?", dbapi_module.version)
            if m:
                version = tuple(
                    int(x) for x in m.group(1, 2, 3) if x is not None
                )
        self.oracledb_ver = version
        if self.oracledb_ver < (1,) and self.oracledb_ver > (0, 0, 0):
            raise exc.InvalidRequestError(
                "oracledb version 1 and above are supported"
            )


dialect = OracleDialect_oracledb



# -*- coding: utf-8 -*-

import os
import iso8601

from xml.etree import cElementTree as eTree
from datetime import datetime


class ProcInutNfeSaver(object):
    def __init__(self, xmls_base_path):
        self.xmls_base_path = xmls_base_path

    def save_proc_inut_nfe_xml(self, xml, order):
        # type: (str, eTree) -> None

        canceled_date = self._get_canceled_date(order)
        order_id = order.get("orderId").zfill(9)

        proc_inut_nfe_xml = eTree.XML(xml)
        serie = proc_inut_nfe_xml.find("{{{0}}}inutNFe/{{{0}}}infInut/{{{0}}}serie".format("http://www.portalfiscal.inf.br/nfe")).text.zfill(3)
        fiscal_number = proc_inut_nfe_xml.find("{{{0}}}inutNFe/{{{0}}}infInut/{{{0}}}nNFIni".format("http://www.portalfiscal.inf.br/nfe")).text.zfill(9)
        file_name = "{0}_{1}_{2}_procInutNfe.xml".format(serie, fiscal_number, order_id)

        save_dir = os.path.join(self.xmls_base_path, "Enviados", canceled_date.strftime("%Y"), canceled_date.strftime("%m"), canceled_date.strftime("%d"))
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)

        save_path = os.path.join(save_dir, file_name)

        with open(save_path, mode="w") as proc_inut_nfe_file:
            proc_inut_nfe_file.write(xml)

    @staticmethod
    def _get_canceled_date(order):
        try:
            canceled_date_str = order.find("StateHistory/State[@state='VOIDED']").get("timestamp")
            canceled_date = iso8601.parse_date(canceled_date_str)
        except Exception as _:
            canceled_date = datetime.now()

        return canceled_date

    def save_proc_inut_nfe_xml_error(self, xml, order):
        # type: (str, eTree) -> None
        order_id = order.get("orderId").zfill(9)

        proc_inut_nfe_xml = eTree.XML(xml)
        serie = proc_inut_nfe_xml.find("{{{0}}}inutNFe/{{{0}}}infInut/{{{0}}}serie".format("http://www.portalfiscal.inf.br/nfe")).text.zfill(3)
        fiscal_number = proc_inut_nfe_xml.find("{{{0}}}inutNFe/{{{0}}}infInut/{{{0}}}nNFIni".format("http://www.portalfiscal.inf.br/nfe")).text.zfill(9)
        file_name = "{0}_{1}_{2}_procInutNfe.xml".format(serie, fiscal_number, order_id)

        save_dir = os.path.join(self.xmls_base_path, "Erros")
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)

        save_path = os.path.join(save_dir, file_name)

        with open(save_path, mode="w") as proc_inut_nfe_file:
            proc_inut_nfe_file.write(xml)



from get_authorization import get_authorization
from list_users import (
    check_valid_user,
    is_user_already_logged,
    list_ids_opened_users,
    list_opened_users,
    set_context,
)
from get_drawer_amount import get_drawer_amount
from get_drawer_initial_amount import get_drawer_initial_amount
from get_pos_user import get_pos_users, get_opened_pos_user
from get_working_type import get_working_type
from get_operator_info import (
    get_current_operator_session,
    get_current_state_session,
    get_operator_info,
    is_current_operator_opened
)
from get_operator_level import get_user_level
from get_operator_name import get_operator_name
from get_payments_amount_by_type import get_payments_amount_by_type
from cash_out_functions import do_set_drawer_status, check_sangria, is_sangria_enable
from delivery_functions import is_delivery_order
from pos_type import set_pos_type, get_pos_types
from get_i18n import get_i18n
from convert_to_localtime import convert_to_localtime
from get_pos_language import get_pos_language
from get_tender_type import (
    get_tender_type,
    get_all_tender_types,
)



from typing import Dict


class L10NRepository(object):

    def __init__(
        self,
        l10n_params: Dict[str, str],
    ) -> None:

        self.l10n_params = l10n_params

    def get_currency_symbol(
        self,
    ) -> str:

        return self.l10n_params["L10N_CURRENCY_SYMBOL"]

    def get_currency_symbol_position(
        self,
    ) -> str:

        return self.l10n_params["L10N_CURRENCY_SYMBOL_POSITION"]

    def get_decimal_places(
        self,
    ) -> str:

        return self.l10n_params["L10N_DECIMAL_PLACES"]

    def get_decimal_separator(
        self,
    ) -> str:

        return self.l10n_params["L10N_DECIMALS_SEPARATOR"]

    def get_thousands_separator(
        self,
    ) -> str:

        return self.l10n_params["L10N_THOUSANDS_SEPARATOR"]

    def get_datetime_format(
        self,
    ) -> str:

        return self.l10n_params["L10_PYTHON_DATETIME_FORMAT"]



from old_helper import BaseRepository
from typing import List


class ServicePeriodRepository(BaseRepository):
    def __init__(self, mb_context):
        super(ServicePeriodRepository, self).__init__(mb_context)

    def get_services_period(self):
        # type: () -> List

        def inner_func(conn):
            services = []
            cursor = conn.select(self._get_service_periods_query)
            if cursor.rows() > 0:
                for row in cursor:
                    data = {'id': row.get_entry("ServicePeriodId"),
                            'name': row.get_entry("Name"),
                            'startHour':  row.get_entry("StartHour"),
                            'endHour': row.get_entry("EndHour"),
                            'weekDays': row.get_entry("WeekDays"),
                            'priceListId': row.get_entry("PriceListId"),
                            'priceListAbbr': row.get_entry("Abbreviation") or ""
                            }
                    services.append(data)

            return services

        return self.execute_with_connection(inner_func)

    _get_service_periods_query = \
        """SELECT * 
             FROM ServicePeriod s"""



import json

from ._Line import Line
from ._LineParser import JSON


class JSONReportParserToHtml(object):

    def parse(
        self,      # type: JSONReportParserToHtml
        report,    # type: str
    ):
        # type: (...) -> str

        html = []
        report = json.loads(report)
        if not report:
            return ""

        columns = 38
        header = report[0].get("header")
        if header is not None:
            columns = header.get("columns", 38)
            report.pop(0)

        self._add_break_line_at_start_and_end(report=report)
        for report_line in report:
            html_line = Line.parse(
                line=report_line,
                columns=columns,
            )
            html.extend(html_line)
            html.append("<br/>")

        return "".join(html)

    @staticmethod
    def _add_break_line_at_start_and_end(
        report,    # type: JSON
    ):
        # type: (...) -> None

        break_line = {
            "command": "{CMD_BREAK_LINE}"
        }
        report.insert(
            0,
            break_line,
        )
        report.extend([
            break_line,
            break_line,
        ])



from sysactions import get_cfg

ask_pager_number = {}


def should_ask_pager_number(
    pos_id,  # type: str
):
    # type: (...) -> bool

    global ask_pager_number

    if pos_id not in ask_pager_number:
        ask_pager_number[str(pos_id)] = get_cfg(pos_id).find_value("AskPagerNumber", "false").lower() == "true"

    return ask_pager_number[pos_id]



import json

from application.interactor import GetEndPointsConfigInteractor
from messagebus import (
    DefaultToken,
    DataType,
    Message,
    MessageBus,
    AckMessage,
    Event,
    InternalErrorMessage,
)
from typing import (
    Any,
    Dict,
)
from ._NoErrorProcessor import NoErrorProcessor


class GetEndPointsConfigProcessor(NoErrorProcessor):
    def __init__(
        self,          # type: GetEndPointsConfigProcessor
        interactor,    # type: GetEndPointsConfigInteractor
    ):
        # type: (...) -> None

        super(GetEndPointsConfigProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(
        self,          # type: GetEndPointsConfigProcessor
        data,          # type: bytes
        event_type,    # type: str
    ):
        # type: (...) -> None

        return

    def call_business(
        self,       # type: GetEndPointsConfigProcessor
        request,    # type: Any
    ):
        # type: (...) -> Dict[str, bool]

        return self.interactor.get_loyalty_endpoints_config()

    def format_response(
        self,           # type: GetEndPointsConfigProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        request,        # type: str
        response,       # type: str
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=AckMessage(
                data_type=DataType.json,
                data=json.dumps(response),
            )
        )



class PosSaleTypeNavigation:
    def __init__(
        self,
        pos_id,             # type: int
        sale_type,          # type: str
        navigation_id,      # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.sale_type = sale_type
        self.navigation_id = navigation_id



from typing import (
    Optional,
    Any,
)

from messageprocessorutil import UuidMessageProcessor
from messagebus import (
    MessageBus,
    Message,
)

from production.interactor import DeviceStatusChangedInteractor


class DeviceStatusChangedProcessor(UuidMessageProcessor):

    def __init__(
        self,                       # type: DeviceStatusChangedProcessor
        device_status_interactor,   # type: DeviceStatusChangedInteractor
    ):
        # type: (...) -> None

        super(DeviceStatusChangedProcessor, self).__init__()

        self.devices_status_interactor = device_status_interactor

    def parse_data(
        self,           # type: DeviceStatusChangedProcessor
        data,           # type: Any
        event_type,     # type: str
    ):
        # type: (...) -> None

        return None

    def call_business(
        self,   # type: DeviceStatusChangedProcessor
        data,   # type: Any
    ):
        # type: (...) -> None

        return self.devices_status_interactor.execute()

    def format_response(
        self,           # type: DeviceStatusChangedProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Optional[Any]
        request,        # type: Any
        result,         # type: Any
    ):
        # type: (...) -> None

        return self._do_not_reply_to_asynchronous_event()

    def format_exception(
        self,           # type: DeviceStatusChangedProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Optional[Any]
        input_obj,      # type: Any
        exception,      # type: Exception
    ):
        # type: (...) -> None

        return self._do_not_reply_to_asynchronous_event()

    def format_parse_exception(
        self,           # type: DeviceStatusChangedProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Optional[Any]
        data,           # type: Any
        exception,      # type: Exception
    ):
        # type: (...) -> None

        return self._do_not_reply_to_asynchronous_event()

    @staticmethod
    def _do_not_reply_to_asynchronous_event():
        # type: (...) -> None

        return None



from ._common import pytz_imported


class PytzUsageWarning(RuntimeWarning):
    """Warning raised when accessing features specific to ``pytz``'s interface.

    This warning is used to direct users of ``pytz``-specific features like the
    ``localize`` and ``normalize`` methods towards using the standard
    ``tzinfo`` interface, so that these shims can be replaced with one of the
    underlying libraries they are wrapping.
    """


class UnknownTimeZoneError(KeyError):
    """Raised when no time zone is found for a specified key."""


class InvalidTimeError(Exception):
    """The base class for exceptions related to folds and gaps."""


class AmbiguousTimeError(InvalidTimeError):
    """Exception raised when ``is_dst=None`` for an ambiguous time (fold)."""


class NonExistentTimeError(InvalidTimeError):
    """Exception raised when ``is_dst=None`` for a non-existent time (gap)."""


PYTZ_BASE_ERROR_MAPPING = {}


def _make_pytz_derived_errors(
    InvalidTimeError_=InvalidTimeError,
    AmbiguousTimeError_=AmbiguousTimeError,
    NonExistentTimeError_=NonExistentTimeError,
    UnknownTimeZoneError_=UnknownTimeZoneError,
):
    if PYTZ_BASE_ERROR_MAPPING or not pytz_imported():
        return

    import pytz

    class InvalidTimeError(InvalidTimeError_, pytz.InvalidTimeError):
        pass

    class AmbiguousTimeError(AmbiguousTimeError_, pytz.AmbiguousTimeError):
        pass

    class NonExistentTimeError(
        NonExistentTimeError_, pytz.NonExistentTimeError
    ):
        pass

    class UnknownTimeZoneError(
        UnknownTimeZoneError_, pytz.UnknownTimeZoneError
    ):
        pass

    PYTZ_BASE_ERROR_MAPPING.update(
        {
            InvalidTimeError_: InvalidTimeError,
            AmbiguousTimeError_: AmbiguousTimeError,
            NonExistentTimeError_: NonExistentTimeError,
            UnknownTimeZoneError_: UnknownTimeZoneError,
        }
    )


def get_exception(exc_type, msg):
    _make_pytz_derived_errors()

    out_exc_type = PYTZ_BASE_ERROR_MAPPING.get(exc_type, exc_type)

    return out_exc_type(msg)



from typing import List
from xml.etree import cElementTree as eTree


class MobileProductParser(object):

    def __init__(
        self,
        product_xml: eTree.Element,
        params: List[str],
    ) -> None:

        self.product_code: str
        self.product_name: str
        self.product_qty: str
        self.product_unit: str
        self.product_price: str
        self.product_total_price: str

        self._parse_and_format_params(
            product_xml=product_xml,
            params=params,
        )

    def _parse_and_format_params(
        self,
        product_xml: eTree.Element,
        params: List[str],
    ) -> None:

        self.product_code = product_xml.find(params[0]).text
        self.product_name = product_xml.find(params[1]).text
        self.product_qty = "{:.3f}".format(float(product_xml.find(params[2]).text))
        self.product_unit = product_xml.find(params[3]).text
        self.product_price = "{:.2f}".format(float(product_xml.find(params[4]).text))
        self.product_total_price = "{:.2f}".format(float(product_xml.find(params[5]).text))



# Copyright The OpenTelemetry Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""OTLP Exporter"""

import threading
from abc import ABC, abstractmethod
from collections.abc import Sequence  # noqa: F401
from logging import getLogger
from os import environ
from time import sleep
from typing import (  # noqa: F401
    Any,
    Callable,
    Dict,
    Generic,
    List,
    Optional,
    Tuple,
    Union,
)
from typing import Sequence as TypingSequence
from typing import TypeVar
from urllib.parse import urlparse

from deprecated import deprecated

from opentelemetry.exporter.otlp.proto.common._internal import (
    _get_resource_data,
    _create_exp_backoff_generator,
)
from google.rpc.error_details_pb2 import RetryInfo
from grpc import (
    ChannelCredentials,
    Compression,
    RpcError,
    StatusCode,
    insecure_channel,
    secure_channel,
    ssl_channel_credentials,
)

from opentelemetry.exporter.otlp.proto.grpc import (
    _OTLP_GRPC_HEADERS,
)
from opentelemetry.proto.common.v1.common_pb2 import (  # noqa: F401
    AnyValue,
    ArrayValue,
    KeyValue,
)
from opentelemetry.proto.resource.v1.resource_pb2 import Resource  # noqa: F401
from opentelemetry.sdk.environment_variables import (
    OTEL_EXPORTER_OTLP_CERTIFICATE,
    OTEL_EXPORTER_OTLP_COMPRESSION,
    OTEL_EXPORTER_OTLP_ENDPOINT,
    OTEL_EXPORTER_OTLP_HEADERS,
    OTEL_EXPORTER_OTLP_INSECURE,
    OTEL_EXPORTER_OTLP_TIMEOUT,
)
from opentelemetry.sdk.metrics.export import MetricsData
from opentelemetry.sdk.resources import Resource as SDKResource
from opentelemetry.sdk.trace import ReadableSpan
from opentelemetry.util.re import parse_env_headers

logger = getLogger(__name__)
SDKDataT = TypeVar("SDKDataT")
ResourceDataT = TypeVar("ResourceDataT")
TypingResourceT = TypeVar("TypingResourceT")
ExportServiceRequestT = TypeVar("ExportServiceRequestT")
ExportResultT = TypeVar("ExportResultT")

_ENVIRON_TO_COMPRESSION = {
    None: None,
    "gzip": Compression.Gzip,
}


class InvalidCompressionValueException(Exception):
    def __init__(self, environ_key: str, environ_value: str):
        super().__init__(
            'Invalid value "{}" for compression envvar {}'.format(
                environ_value, environ_key
            )
        )


def environ_to_compression(environ_key: str) -> Optional[Compression]:
    environ_value = (
        environ[environ_key].lower().strip()
        if environ_key in environ
        else None
    )
    if environ_value not in _ENVIRON_TO_COMPRESSION:
        raise InvalidCompressionValueException(environ_key, environ_value)
    return _ENVIRON_TO_COMPRESSION[environ_value]


@deprecated(
    version="1.18.0",
    reason="Use one of the encoders from opentelemetry-exporter-otlp-proto-common instead",
)
def get_resource_data(
    sdk_resource_scope_data: Dict[SDKResource, ResourceDataT],
    resource_class: Callable[..., TypingResourceT],
    name: str,
) -> List[TypingResourceT]:
    return _get_resource_data(sdk_resource_scope_data, resource_class, name)


def _load_credential_from_file(filepath) -> ChannelCredentials:
    try:
        with open(filepath, "rb") as creds_file:
            credential = creds_file.read()
            return ssl_channel_credentials(credential)
    except FileNotFoundError:
        logger.exception("Failed to read credential file")
        return None


def _get_credentials(creds, environ_key):
    if creds is not None:
        return creds
    creds_env = environ.get(environ_key)
    if creds_env:
        return _load_credential_from_file(creds_env)
    return ssl_channel_credentials()


# pylint: disable=no-member
class OTLPExporterMixin(
    ABC, Generic[SDKDataT, ExportServiceRequestT, ExportResultT]
):
    """OTLP span exporter

    Args:
        endpoint: OpenTelemetry Collector receiver endpoint
        insecure: Connection type
        credentials: ChannelCredentials object for server authentication
        headers: Headers to send when exporting
        timeout: Backend request timeout in seconds
        compression: gRPC compression method to use
    """

    def __init__(
        self,
        endpoint: Optional[str] = None,
        insecure: Optional[bool] = None,
        credentials: Optional[ChannelCredentials] = None,
        headers: Optional[
            Union[TypingSequence[Tuple[str, str]], Dict[str, str], str]
        ] = None,
        timeout: Optional[int] = None,
        compression: Optional[Compression] = None,
    ):
        super().__init__()

        self._endpoint = endpoint or environ.get(
            OTEL_EXPORTER_OTLP_ENDPOINT, "http://localhost:4317"
        )

        parsed_url = urlparse(self._endpoint)

        if parsed_url.scheme == "https":
            insecure = False
        if insecure is None:
            insecure = environ.get(OTEL_EXPORTER_OTLP_INSECURE)
            if insecure is not None:
                insecure = insecure.lower() == "true"
            else:
                if parsed_url.scheme == "http":
                    insecure = True
                else:
                    insecure = False

        if parsed_url.netloc:
            self._endpoint = parsed_url.netloc

        self._headers = headers or environ.get(OTEL_EXPORTER_OTLP_HEADERS)
        if isinstance(self._headers, str):
            temp_headers = parse_env_headers(self._headers)
            self._headers = tuple(temp_headers.items())
        elif isinstance(self._headers, dict):
            self._headers = tuple(self._headers.items())
        if self._headers is None:
            self._headers = tuple(_OTLP_GRPC_HEADERS)
        else:
            self._headers = tuple(self._headers) + tuple(_OTLP_GRPC_HEADERS)

        self._timeout = timeout or int(
            environ.get(OTEL_EXPORTER_OTLP_TIMEOUT, 10)
        )
        self._collector_kwargs = None

        compression = (
            environ_to_compression(OTEL_EXPORTER_OTLP_COMPRESSION)
            if compression is None
            else compression
        ) or Compression.NoCompression

        if insecure:
            self._client = self._stub(
                insecure_channel(self._endpoint, compression=compression)
            )
        else:
            credentials = _get_credentials(
                credentials, OTEL_EXPORTER_OTLP_CERTIFICATE
            )
            self._client = self._stub(
                secure_channel(
                    self._endpoint, credentials, compression=compression
                )
            )

        self._export_lock = threading.Lock()
        self._shutdown = False

    @abstractmethod
    def _translate_data(
        self, data: TypingSequence[SDKDataT]
    ) -> ExportServiceRequestT:
        pass

    def _export(
        self, data: Union[TypingSequence[ReadableSpan], MetricsData]
    ) -> ExportResultT:
        # After the call to shutdown, subsequent calls to Export are
        # not allowed and should return a Failure result.
        if self._shutdown:
            logger.warning("Exporter already shutdown, ignoring batch")
            return self._result.FAILURE

        # FIXME remove this check if the export type for traces
        # gets updated to a class that represents the proto
        # TracesData and use the code below instead.
        # logger.warning(
        #     "Transient error %s encountered while exporting %s, retrying in %ss.",
        #     error.code(),
        #     data.__class__.__name__,
        #     delay,
        # )
        max_value = 64
        # expo returns a generator that yields delay values which grow
        # exponentially. Once delay is greater than max_value, the yielded
        # value will remain constant.
        for delay in _create_exp_backoff_generator(max_value=max_value):
            if delay == max_value or self._shutdown:
                return self._result.FAILURE

            with self._export_lock:
                try:
                    self._client.Export(
                        request=self._translate_data(data),
                        metadata=self._headers,
                        timeout=self._timeout,
                    )

                    return self._result.SUCCESS

                except RpcError as error:

                    if error.code() in [
                        StatusCode.CANCELLED,
                        StatusCode.DEADLINE_EXCEEDED,
                        StatusCode.RESOURCE_EXHAUSTED,
                        StatusCode.ABORTED,
                        StatusCode.OUT_OF_RANGE,
                        StatusCode.UNAVAILABLE,
                        StatusCode.DATA_LOSS,
                    ]:

                        retry_info_bin = dict(error.trailing_metadata()).get(
                            "google.rpc.retryinfo-bin"
                        )
                        if retry_info_bin is not None:
                            retry_info = RetryInfo()
                            retry_info.ParseFromString(retry_info_bin)
                            delay = (
                                retry_info.retry_delay.seconds
                                + retry_info.retry_delay.nanos / 1.0e9
                            )

                        logger.warning(
                            (
                                "Transient error %s encountered while exporting "
                                "%s to %s, retrying in %ss."
                            ),
                            error.code(),
                            self._exporting,
                            self._endpoint,
                            delay,
                        )
                        sleep(delay)
                        continue
                    else:
                        logger.error(
                            "Failed to export %s to %s, error code: %s",
                            self._exporting,
                            self._endpoint,
                            error.code(),
                        )

                    if error.code() == StatusCode.OK:
                        return self._result.SUCCESS

                    return self._result.FAILURE

        return self._result.FAILURE

    def shutdown(self, timeout_millis: float = 30_000, **kwargs) -> None:
        if self._shutdown:
            logger.warning("Exporter already shutdown, ignoring call")
            return
        # wait for the last export if any
        self._export_lock.acquire(timeout=timeout_millis)
        self._shutdown = True
        self._export_lock.release()

    @property
    @abstractmethod
    def _exporting(self) -> str:
        """
        Returns a string that describes the overall exporter, to be used in
        warning messages.
        """
        pass



import json
import logging

from application.model import ApplierSubTypes
from application.repository.api import APIRepository
from typing import Optional

logger = logging.getLogger(__name__)


class GetLoyaltyCustomerInfoInteractor(object):
    def __init__(self, api_repository):
        # type: (APIRepository) -> None

        self.api_repository = api_repository

    def get_loyalty_customer_info(
        self,                 # type: GetLoyaltyCustomerInfoInteractor
        loyalty_customer_id,  # type: str
    ):
        # type: (...) -> Optional[str]

        logger.info("Requesting customer loyaltySystemId")
        loyalty_info = self.api_repository.retrieve_loyalty_customer_info(loyalty_customer_id=loyalty_customer_id)
        logger.info("Retrieved customer loyalty data: {}".format(loyalty_info))
        loyalty_info = self._update_applier_type(customer_info=loyalty_info)
        return loyalty_info

    @staticmethod
    def _update_applier_type(
        customer_info,  # type: str
    ):
        # type: (...) -> str

        if not customer_info:
            return customer_info

        try:
            info = json.loads(customer_info)
            benefits = info.get("benefits", [])
            for benefit in benefits:
                benefit["applier"] = "Loyalty"
                benefit["applierSubType"] = ApplierSubTypes.LOYALTY

            return json.dumps(info)
        except ValueError:
            logger.error("Invalid JSON returned: {}".format(customer_info))
            return customer_info



"""
Launch the Python script on the command line after
setuptools is bootstrapped via import.
"""

# Note that setuptools gets imported implicitly by the
# invocation of this script using python -m setuptools.launch

import sys
import tokenize


def run():
    """
    Run the script in sys.argv[1] as if it had
    been invoked naturally.
    """
    __builtins__
    script_name = sys.argv[1]
    namespace = dict(
        __file__=script_name,
        __name__='__main__',
        __doc__=None,
    )
    sys.argv[:] = sys.argv[1:]

    open_ = getattr(tokenize, 'open', open)
    with open_(script_name) as fid:
        script = fid.read()
    norm_script = script.replace('\\r\\n', '\\n')
    code = compile(norm_script, script_name, 'exec')
    exec(code, namespace)


if __name__ == '__main__':
    run()



from abc import abstractmethod
from typing import Sequence


class FileSystem(object):

    @abstractmethod
    def list_entries(
        self,
        path: str,
    ) -> Sequence[str]:

        raise NotImplementedError()

    def read_file_content(
        self,
        path: str,
    ) -> bytes:

        raise NotImplementedError()

    def is_dir(
        self,
        path: str,
    ) -> bool:

        raise NotImplementedError()

    def is_file(
        self,
        path: str,
    ) -> bool:

        raise NotImplementedError()



import os
import sys

from typing import Union


if sys.version_info >= (3, 9):
    StrPath = Union[str, os.PathLike[str]]
else:
    # PathLike is only subscriptable at runtime in 3.9+
    StrPath = Union[str, "os.PathLike[str]"]



import hashlib
from datetime import datetime

from pos_model import Order
from typing import Dict, Any

from _ContextKeys import ContextKeys
from _NfceXmlPartBuilder import NfceXmlPartBuilder


class InfNFeSuplBuilder(NfceXmlPartBuilder):

    def __init__(self, ambiente, cid_token, csc, qrcode_base_url, qrcode_check_url, versao_ws):
        self.environment_identifier = ambiente
        self.csc_identifier = cid_token
        self.csc = csc
        self.qrcode_base_url = qrcode_base_url
        self.qrcode_check_url = qrcode_check_url
        self.ws_version = versao_ws

    def build_xml(self, order, context):
        # type: (Order, Dict[unicode, Any]) -> unicode

        qrcode_version = 1 if self.ws_version in (0, 1, 3) else 2
        access_key = context[ContextKeys.nfce_key]
        emission_date = context[ContextKeys.data_emissao]
        nf_value = context[ContextKeys.v_nf] # type: float
        icms_value = context[ContextKeys.total_icms] # type: float
        digest_value = context[ContextKeys.digest_value].encode("hex")

        if self.ws_version in (0, 1, 3):
            cdata = self._get_cdata_for_nfce_1_or_3(context, emission_date, digest_value, access_key, nf_value, icms_value)
        elif self.ws_version == 4:
            cdata = self._get_cdata_for_nfce_4(context, emission_date, digest_value, access_key, qrcode_version, nf_value)
        else:
            raise Exception("Version of NFCE not supported. ws_version: {}".format(self.ws_version))

        qrcode = u"<qrCode><![CDATA[{0}]]></qrCode>".format(cdata)
        if self.ws_version == 4:
            qrcode += u"<urlChave>{0}</urlChave>".format(self.qrcode_check_url)
        infnfe_supl = u"<infNFeSupl>{0}</infNFeSupl>".format(qrcode)

        return infnfe_supl

    def _get_cdata_for_nfce_1_or_3(self, context, emission_date, digest_value, access_key, nf_value, icms_value):
        # type: (Dict[unicode, Any], unicode, unicode, unicode, float, float) -> unicode

        hash_params = u"chNFe={0}&nVersao=100&tpAmb={1}".format(access_key, self.environment_identifier)
        hash_params = self._get_cpf_cnpj(context, hash_params)
        hash_params += u"&dhEmi={0}&vNF={1:>0.2f}&vICMS={2:>0.2f}&digVal={3}&cIdToken={4}" \
            .format(emission_date.encode("hex"), nf_value, icms_value, digest_value, self.csc_identifier)

        hash_qrcode = hashlib.sha1(hash_params + self.csc).hexdigest()
        hash_params += u"&cHashQRCode={0}".format(hash_qrcode)
        cdata = self.qrcode_base_url + hash_params

        return cdata

    def _get_cdata_for_nfce_4(self, context, emission_date, digest_value, access_key, qrcode_version, nf_value):
        # type: (Dict[unicode, Any], unicode, unicode, unicode, int, float) -> unicode

        if not context[ContextKeys.is_in_contingency]:
            hash_params = u"{0}|{1}|{2}|{3}".format(access_key, qrcode_version, self.environment_identifier, int(self.csc_identifier))

            hash_qrcode = hashlib.sha1(hash_params + self.csc).hexdigest()
            cdata = self._create_cdata_online(hash_qrcode, access_key, qrcode_version)
        else:
            nf_value = "%.02f" % nf_value
            emission_day = datetime.strptime(emission_date.split("T")[0], "%Y-%m-%d").day
            emission_day = str(emission_day).zfill(2)
            hash_params = u"{0}|{1}|{2}|{3}|{4}|{5}|{6}".format(access_key, qrcode_version, self.environment_identifier,
                                                                emission_day, nf_value, digest_value, int(self.csc_identifier))

            hash_qrcode = hashlib.sha1(hash_params + self.csc).hexdigest()
            cdata = self._create_cdata_offline(hash_qrcode, access_key, qrcode_version, emission_day, nf_value, digest_value)

        return cdata

    def _create_cdata_online(self, hash_qrcode, access_key, qrcode_version):
        # type: (unicode, unicode, int) -> unicode

        cdata = u"{url_sefaz}p={chave_de_acesso}|" \
                u"{versao_do_qr_code}|" \
                u"{tipo_de_ambiente}|" \
                u"{identificador_do_csc}|" \
                u"{codigo_hash}" \
            .format(url_sefaz=self.qrcode_base_url,
                    chave_de_acesso=access_key,
                    versao_do_qr_code=qrcode_version,
                    tipo_de_ambiente=self.environment_identifier,
                    identificador_do_csc=int(self.csc_identifier),
                    codigo_hash=hash_qrcode.upper())

        return cdata

    def _create_cdata_offline(self, hash_qrcode, access_key, qrcode_version, emission_day, nf_value, digest_value):
        # type: (unicode, unicode, int, unicode, unicode, unicode) -> unicode

        cdata = u"{url_sefaz}p={chave_de_acesso}|" \
                u"{versao_do_qr_code}|" \
                u"{tipo_de_ambiente}|" \
                u"{dia_emissao}|" \
                u"{valor_total_nfce}|" \
                u"{digest_value}|" \
                u"{identificador_do_csc}|" \
                u"{codigo_hash}" \
            .format(url_sefaz=self.qrcode_base_url,
                    chave_de_acesso=access_key,
                    versao_do_qr_code=qrcode_version,
                    tipo_de_ambiente=self.environment_identifier,
                    dia_emissao=emission_day,
                    valor_total_nfce=nf_value,
                    digest_value=digest_value,
                    identificador_do_csc=int(self.csc_identifier),
                    codigo_hash=hash_qrcode.upper())

        return cdata

    @staticmethod
    def _get_cpf_cnpj(context, hash_params):
        # type: (Dict[unicode, Any], unicode) -> unicode

        cpf_cnpj = None
        if ContextKeys.cpf_cnpj in context:
            cpf_cnpj = context[ContextKeys.cpf_cnpj]

        if cpf_cnpj is not None:
            hash_params += u"&cDest=" + cpf_cnpj

        return hash_params


from ._CustomerCallCouponBuilder import CustomerCallCouponBuilder
from ._CustomerCallCouponMobileLargeBuilder import CustomerCallCouponMobileLargeBuilder
from ._CustomerCallCouponMobileSmallBuilder import CustomerCallCouponMobileSmallBuilder



import sysactions

from .. import logger

from .. import mb_context


@sysactions.action
def open_delivery_store(pos_id, store_id):
    try:
        mb_context.MB_EasyEvtSend("OpenDeliveryStore", type="", xml=store_id, sourceid=int(pos_id))

    except Exception as e:
        sysactions.show_messagebox(pos_id, "$ERROR_OPENING_DELIVERY_STORE")
        logger.error(e)
        return False

    return True



"""Tests for distutils.command.build_scripts."""

import os
import textwrap
from distutils import sysconfig
from distutils.command.build_scripts import build_scripts
from distutils.core import Distribution
from distutils.tests import support

import jaraco.path


class TestBuildScripts(support.TempdirManager):
    def test_default_settings(self):
        cmd = self.get_build_scripts_cmd("/foo/bar", [])
        assert not cmd.force
        assert cmd.build_dir is None

        cmd.finalize_options()

        assert cmd.force
        assert cmd.build_dir == "/foo/bar"

    def test_build(self):
        source = self.mkdtemp()
        target = self.mkdtemp()
        expected = self.write_sample_scripts(source)

        cmd = self.get_build_scripts_cmd(
            target, [os.path.join(source, fn) for fn in expected]
        )
        cmd.finalize_options()
        cmd.run()

        built = os.listdir(target)
        for name in expected:
            assert name in built

    def get_build_scripts_cmd(self, target, scripts):
        import sys

        dist = Distribution()
        dist.scripts = scripts
        dist.command_obj["build"] = support.DummyCommand(
            build_scripts=target, force=True, executable=sys.executable
        )
        return build_scripts(dist)

    @staticmethod
    def write_sample_scripts(dir):
        spec = {
            'script1.py': textwrap.dedent("""
                #! /usr/bin/env python2.3
                # bogus script w/ Python sh-bang
                pass
                """).lstrip(),
            'script2.py': textwrap.dedent("""
                #!/usr/bin/python
                # bogus script w/ Python sh-bang
                pass
                """).lstrip(),
            'shell.sh': textwrap.dedent("""
                #!/bin/sh
                # bogus shell script w/ sh-bang
                exit 0
                """).lstrip(),
        }
        jaraco.path.build(spec, dir)
        return list(spec)

    def test_version_int(self):
        source = self.mkdtemp()
        target = self.mkdtemp()
        expected = self.write_sample_scripts(source)

        cmd = self.get_build_scripts_cmd(
            target, [os.path.join(source, fn) for fn in expected]
        )
        cmd.finalize_options()

        # https://bugs.python.org/issue4524
        #
        # On linux-g++-32 with command line `./configure --enable-ipv6
        # --with-suffix=3`, python is compiled okay but the build scripts
        # failed when writing the name of the executable
        old = sysconfig.get_config_vars().get('VERSION')
        sysconfig._config_vars['VERSION'] = 4
        try:
            cmd.run()
        finally:
            if old is not None:
                sysconfig._config_vars['VERSION'] = old

        built = os.listdir(target)
        for name in expected:
            assert name in built



from datetime import (
    datetime,
    timedelta,
)
from random import randint
from typing import Optional

from application.interactor import LoaderUpdaterInteractor
from application.manager._UpdaterManager import UpdaterManager
from application.model import UpdateType
from application.model.configuration import Configurations
from application.model.customexception import NoUpdateFound
from application.repository.pos import (
    LoaderUpdaterPOSRepository,
    CatalogUpdaterPOSRepository,
    BinaryUpdaterPOSRepository,
    CommonPOSRepository,
)
from application.service import LoaderUpdaterService
from application.util import LoggerUtil

logger = LoggerUtil.get_logger_name(__name__)


class LoaderUpdaterManager(UpdaterManager):
    def __init__(
        self,
        configs,                    # type: Configurations
        interactor,                 # type: LoaderUpdaterInteractor
        pos_repository,             # type: LoaderUpdaterPOSRepository
        catalog_pos_repository,     # type: CatalogUpdaterPOSRepository
        binaries_repository,        # type: BinaryUpdaterPOSRepository
        loader_updater_service,     # type: LoaderUpdaterService
        common_pos_repository,      # type: CommonPOSRepository
    ):
        # type: (...) -> None # noqa

        name = UpdateType.loader.name
        super(LoaderUpdaterManager, self).__init__(name, configs, interactor, pos_repository, logger)
        self.interactor = interactor
        self.catalog_pos_repository = catalog_pos_repository
        self.binaries_repository = binaries_repository
        self.configs = configs
        self.pos_repository = pos_repository
        self.loader_updater_service = loader_updater_service
        self.common_pos_repository = common_pos_repository

        self.update_value = UpdateType.loader.value
        self.logger = logger

        try:
            if not self.has_already_applied_update():
                self.has_update_to_apply()
                self.update()
        except (Exception,):
            self.logger.exception("Error updating loader on manager")

    def update(self):
        # type: () -> None

        self.interactor.update_loaders()

    def has_update_to_apply(
        self,   # type: LoaderUpdaterManager
    ):
        # type: (...) -> None

        if not self.binaries_repository.has_already_applied_update():
            raise NoUpdateFound()

        if not self.catalog_pos_repository.has_already_applied_update():
            raise NoUpdateFound()

        last_update_date = self.pos_repository.get_last_loader_update_date()
        today = datetime.today().replace(hour=0, minute=0, second=0, microsecond=0)
        update_to_apply = last_update_date is None or last_update_date < today
        if not update_to_apply:
            raise NoUpdateFound()

    def notify_pending_update(
        self,   # type: LoaderUpdaterManager
    ):
        # type: (...) -> None

        has_pending_notify = self.pos_repository.has_pending_loader_update_to_notify()
        if not has_pending_notify:
            return

        last_loader_update = self.pos_repository.get_last_loader_update()
        if last_loader_update is None:
            return

        self.loader_updater_service.send_result_message(
            current_update_id=str(last_loader_update.update_id),
        )

    def get_pending_remote_update_id(
        self,   # type: LoaderUpdaterManager
    ):
        # type: (...) -> Optional[str]

        return self.pos_repository.get_pending_loader_external_update_id()

    def get_pending_remote_update_to_schedule(
        self,                  # type: LoaderUpdaterManager
        external_update_id,    # type: Optional[str]
    ):
        # type: (...) -> Optional[datetime.time]

        update_time = self._get_random_time()
        if not update_time:
            return

        if not external_update_id or external_update_id in self.update_scheduled:
            return

        self.update_scheduled[external_update_id] = update_time
        return update_time

    def has_already_applied_update(self):
        # type: () -> bool

        return self.common_pos_repository.has_already_applied_update(self.configs.persistence_name, self.update_value)

    def _get_random_time(
        self,   # type: LoaderUpdaterManager
    ):
        # type: (...) -> Optional[datetime.time]

        loader = self.configs.updaters.get("loader")
        if not getattr(loader, "remote_update_window", None):
            return

        start_datetime = self._to_datetime(time_to_format=loader.remote_update_window.start_time)
        end_datetime = self._to_datetime(time_to_format=loader.remote_update_window.end_time)
        return self._get_random_time_interval(
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )

    @staticmethod
    def _to_datetime(
        time_to_format,   # type: datetime.time
    ):
        # type: (...) -> datetime

        return datetime.combine(datetime.today(), time_to_format)

    @staticmethod
    def _get_random_time_interval(
        start_datetime,    # type: datetime
        end_datetime,      # type: datetime
    ):
        # type: (...) -> datetime.time

        today = datetime.today()
        if start_datetime.time() < today.time() < end_datetime.time():
            start_datetime = today

        time_interval = (end_datetime - start_datetime).seconds
        random_second = randint(0, time_interval)
        update_time = start_datetime + timedelta(seconds=random_second)

        if update_time < today:
            return update_time + timedelta(days=1)

        return update_time



import logging
from sqlite3 import Connection
from typing import (
    Dict,
    List,
)

from message_handler_builder import Configs
from messagebus import (
    Event,
    Message,
    MessageBus,
)
from messagehandler import MessageHandlerBuilder
from messageprocessor import (
    DefaultMessageProcessorExecutorFactory,
    MessageProcessorMessageHandler,
)
from messageprocessorutil import LoggingProcessorCallback
from model.repository import (
    SqliteConnectionTransactionCallback,
    SqliteRepositoryFactory,
)
from nreports.builders import DataBuilderFactory
from nreports.interactor.build_report import BuildReportInteractor
from nreports.interactor.report_json_parser import ReportJsonParserInteractor
from nreports.processor import (
    BuildReportProcessor,
    ReportJsonParserProcessorBase,
)
from nreports.repository import (
    I18N,
    OrderAPIRepository,
    POSControllerRepository,
    StoreConfigRepository,
    UserRepository,
)
from nreports_api.shared.token import (
    TK_NREPORTS_BUILD_REPORT,
    TK_NREPORTS_REPORT_JSON_PARSER,
)

logger = logging.getLogger(name=__name__)


class NReportsMessageHandlerBuilder(MessageHandlerBuilder):

    def __init__(
        self,
        configs: Configs,
        message_bus: MessageBus,
    ) -> None:

        self.configs = configs
        self.message_bus = message_bus

        self.conn_mapping: Dict[MessageProcessorMessageHandler, List[Connection]] = {}
        self.sqlite_repository_factory = SqliteRepositoryFactory(database_dir=self.configs.database_dir)

        self.i18n_repository: I18N
        self.order_api_repository: OrderAPIRepository
        self.store_config_repository: StoreConfigRepository
        self.pos_controller_repository: POSControllerRepository

    def build_singletons(
        self,
    ) -> None:

        self._create_repositories()
        logger.info("Component started!")

    def build_message_handler(
        self,
        message: Message,
        event: Event,
    ) -> MessageProcessorMessageHandler:

        connections: List[Connection] = []

        data_builder_factory = DataBuilderFactory(
            message_bus=self.message_bus,
            order_api_repository=self.order_api_repository,
            store_config_repository=self.store_config_repository,
            i18n_repository=self.i18n_repository,
            extra_modules=self.configs.builder_extra_modules,
            images_path=self.configs.images_path,
            constants=self.configs.constants,
            user_repository=self.user_repository,
        )
        build_report_interactor = BuildReportInteractor(
            report_files=self.configs.report_files,
            data_builder_factory=data_builder_factory,
            i18n_repository=self.i18n_repository,
        )
        report_json_parser_interactor = ReportJsonParserInteractor(i18n_repository=self.i18n_repository)
        build_report_processor = BuildReportProcessor(interactor=build_report_interactor)
        report_json_parser_processor = ReportJsonParserProcessorBase(interactor=report_json_parser_interactor)

        message_handler = MessageProcessorMessageHandler(
            event_processors={},
            message_processors={
                TK_NREPORTS_BUILD_REPORT: build_report_processor,
                TK_NREPORTS_REPORT_JSON_PARSER: report_json_parser_processor,
            },
            executor_factory=DefaultMessageProcessorExecutorFactory(
                callbacks=[
                    LoggingProcessorCallback(logger=logger),
                    SqliteConnectionTransactionCallback(connections=connections),
                ]
            ),
        )

        self.conn_mapping[message_handler] = connections

        return message_handler

    def _create_repositories(
        self,
    ) -> None:

        self.order_api_repository = OrderAPIRepository(message_bus=self.message_bus)
        self.store_config_repository = StoreConfigRepository(message_bus=self.message_bus)
        self.pos_controller_repository = POSControllerRepository(message_bus=self.message_bus)
        self.user_repository = UserRepository(message_bus=self.message_bus)
        self.i18n_repository = I18N(
            message_bus=self.message_bus,
            pos_controller_repository=self.pos_controller_repository,
        )

    def destroy_message_handler(
        self,
        message_handler: MessageProcessorMessageHandler,
    ) -> None:

        if message_handler in self.conn_mapping:
            connections = self.conn_mapping[message_handler]
            for conn in connections:
                conn.close()
            del self.conn_mapping[message_handler]

    def destroy_singletons(
        self,
    ) -> None:

        logger.info("Component stopped!")



# hetu.py - functions for handling Finnish personal identity codes
# coding: utf-8
#
# Copyright (C) 2011 Jussi Judin
# Copyright (C) 2012, 2013 Arthur de Jong
# Copyright (C) 2020 Aleksi Hoffman
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""HETU (Henkilötunnus, Finnish personal identity code).

Module for handling Finnish personal identity codes (HETU, Henkilötunnus).
See https://www.vaestorekisterikeskus.fi/default.aspx?id=45 for checksum
calculation details and https://tarkistusmerkit.teppovuori.fi/tarkmerk.htm#hetu1
for historical details.

>>> validate('131052-308T')
'131052-308T'
>>> validate('131052-308U')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('310252-308Y')
Traceback (most recent call last):
    ...
InvalidComponent: ...
>>> compact('131052a308t')
'131052A308T'
"""

import datetime
import re

from stdnum.exceptions import *
from stdnum.util import clean


_century_codes = {
    '+': 1800,
}
_century_codes.update(dict.fromkeys(('-', 'Y', 'X', 'W', 'V', 'U'), 1900))
_century_codes.update(dict.fromkeys(('A', 'B', 'C', 'D', 'E', 'F'), 2000))

# Finnish personal identity codes are composed of date part, century
# indicating sign, individual number and control character.
# ddmmyyciiiC
_hetu_re = re.compile(r'^(?P<day>[0123]\d)(?P<month>[01]\d)(?P<year>\d\d)'
                      r'(?P<century>[-+ABCDEFYXWVU])(?P<individual>\d\d\d)'
                      r'(?P<control>[0-9ABCDEFHJKLMNPRSTUVWXY])$')


def compact(number):
    """Convert the HETU to the minimal representation. This strips
    surrounding whitespace and converts it to upper case."""
    return clean(number, '').upper().strip()


def _calc_checksum(number):
    return '0123456789ABCDEFHJKLMNPRSTUVWXY'[int(number) % 31]


def validate(number, allow_temporary=False):
    """Check if the number is a valid HETU. It checks the format, whether a
    valid date is given and whether the check digit is correct. Allows
    temporary identifier range for individuals (900-999) if allow_temporary
    is True.
    """
    number = compact(number)
    match = _hetu_re.search(number)
    if not match:
        raise InvalidFormat()
    day = int(match.group('day'))
    month = int(match.group('month'))
    year = int(match.group('year'))
    century = _century_codes[match.group('century')]
    individual = int(match.group('individual'))
    # check if birth date is valid
    try:
        datetime.date(century + year, month, day)
    except ValueError:
        raise InvalidComponent()
    # for historical reasons individual IDs start from 002
    if individual < 2:
        raise InvalidComponent()
    # this range is for temporary identifiers
    if 900 <= individual <= 999 and not allow_temporary:
        raise InvalidComponent()
    checkable_number = '%02d%02d%02d%03d' % (day, month, year, individual)
    if match.group('control') != _calc_checksum(checkable_number):
        raise InvalidChecksum()
    return number


def is_valid(number, allow_temporary=False):
    """Check if the number is a valid HETU."""
    try:
        return bool(validate(number, allow_temporary))
    except ValidationError:
        return False


# This is here just for completeness as there are no different length forms
# of Finnish personal identity codes:
format = compact



from mw_helper import ensure_iterable
from production.box._OrderChangerProductionBox import OrderChangerProductionBox


class AllItemHasAtLeastOneTagFilterBox(OrderChangerProductionBox):
    def __init__(self, name, sons, tags, logger=None):
        super(AllItemHasAtLeastOneTagFilterBox, self).__init__(name, sons, logger)
        self.tags = ensure_iterable(tags)

    def change_order(self, order):
        new_items = []
        for item in order.items:
            if item.all_has_at_least_one_tag(self.tags):
                new_items.append(item)

        order.items = new_items
        return order



import os

from msgbus import MBEasyContext

from helper import import_pydevd
from mbcontextmessagehandler import (
    MbContextMessageBus,
    MbContextMessageHandler,
)
from externals.messagehandlerbuilder import AlertsMessageHandlerBuilder
from systools import sys_parsefilepath

from services.model import all_events
from externals.configurator import Configurator


def main():
    # type: (...) -> None

    loader_path = os.environ["LOADERCFG"]

    import_pydevd(loader_path=loader_path, port=9184)

    data_path = sys_parsefilepath("{HVDATADIR}")
    databases_path = os.path.join(data_path, "databases")
    backup_databases_path = os.path.join(data_path, "backups_alerts", "databases")

    configurator = Configurator(
        loader_path=loader_path,
        databases_path=databases_path,
        backup_databases_path=backup_databases_path,
    )
    configs = configurator.read_loader_configs()
    logger = configurator.get_loggers()

    mb_context = MBEasyContext(name="Alerts")
    message_bus = MbContextMessageBus(mbcontext=mb_context)

    message_handler_builder = AlertsMessageHandlerBuilder(
        configs=configs,
        message_bus=message_bus,
        mb_context=mb_context,
        logger=logger,
        input_logger=logger,
        callback_logger=logger,
    )
    message_handler = MbContextMessageHandler(
        message_bus=message_bus,
        service_name="Alerts",
        service_type="Alerts",
        required_services="",
        event_handler_builder=message_handler_builder,
    )

    message_handler.subscribe_reentrant_events(all_events())

    logger.info("Alerts started...")

    return message_handler.handle_events()



import importlib
import os
from typing import (
    Dict,
    Optional,
    Sequence,
)

from messagebus import MessageBus
from model.exceptions import (
    ErrorBuildingDataBuilder,
    ReportNotImplemented,
)
from nreports.repository import (
    I18N,
    OrderAPIRepository,
    StoreConfigRepository,
    UserRepository,
)

from ._DataBuilder import DataBuilder


class DataBuilderFactory(object):

    def __init__(
        self,
        message_bus: MessageBus,
        order_api_repository: OrderAPIRepository,
        store_config_repository: StoreConfigRepository,
        i18n_repository: I18N,
        extra_modules: Optional[Sequence[str]],
        images_path: Dict[str, str],
        constants: Dict[str, str],
        user_repository: UserRepository,
    ) -> None:

        self.message_bus = message_bus
        self.order_api_repository = order_api_repository
        self.store_config_repository = store_config_repository
        self.store_config_repository = store_config_repository
        self.user_repository = user_repository
        self.message_bus = message_bus
        self.extra_modules = extra_modules or []
        self.i18n_repository = i18n_repository
        self.images_path = images_path
        self.constants = constants

        self.all_modules = ["nreports.builders"] + list(self.extra_modules)

    def build(
        self,
        report_name: str,
    ) -> DataBuilder:

        self._append_modules_recursively()

        for module_path in self.all_modules:
            try:
                module = importlib.import_module(module_path)
                class_name = self._to_pascal_case(snake_str=report_name) + "Builder"

                data_builder = getattr(module, class_name)(
                    self.message_bus,
                    self.order_api_repository,
                    self.store_config_repository,
                    self.i18n_repository,
                    self.images_path,
                    self.constants,
                    self.user_repository,
                )
                return data_builder

            except (AttributeError, ModuleNotFoundError):
                pass

            except Exception as ex:
                raise ErrorBuildingDataBuilder(
                    report_name=report_name,
                    error_message=str(ex),
                )

        raise ReportNotImplemented(report_name=report_name)

    def _append_modules_recursively(
        self,
    ) -> None:

        current_dir = os.path.dirname(__file__)
        for root, dirs, files in os.walk(current_dir):
            for folder in dirs:
                if folder == "__pycache__":
                    continue

                if os.path.split(root)[1] == "builders":
                    self.all_modules.append(f"nreports.builders.{folder}")
                else:
                    builders_folders = os.path.abspath(root).split("builders\\")
                    if len(builders_folders) < 2:
                        continue

                    module_full_path = builders_folders[1].replace("\\", ".")
                    self.all_modules.append(f"nreports.builders.{module_full_path}.{folder}")

    @staticmethod
    def _to_pascal_case(
        snake_str: str
    ) -> str:

        components = snake_str.split("_")
        return "".join(x.lower().title() for x in components)



import json

from messagebus import (
    DefaultToken,
    DataType,
    Message,
    MessageBus,
    AckMessage,
    Event,
    InternalErrorMessage,
)
from application.interactor.totaled import GetTotaledMessageInteractor
from application.interactor.totaled.dto import GetTotaledMessageRequest
from application.model import TotaledMessage
from typing import Optional
from ._NoErrorProcessor import NoErrorProcessor


class GetTotaledMessageProcessor(NoErrorProcessor):
    def __init__(
        self,          # type: GetTotaledMessageProcessor
        interactor,    # type: GetTotaledMessageInteractor
    ):
        # type: (...) -> None

        super(GetTotaledMessageProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(
        self,          # type: GetTotaledMessageProcessor
        data,          # type: bytes
        event_type,    # type: str
    ):
        # type: (...) -> GetTotaledMessageRequest

        request_json = json.loads(data)

        return GetTotaledMessageRequest(
            pos_id=request_json.get("posId"),
            customer_id=request_json.get("customerId"),
        )

    def call_business(
        self,         # type: GetTotaledMessageProcessor
        request,      # type: GetTotaledMessageRequest
    ):
        # type: (...) -> TotaledMessage

        return self.interactor.get_totaled_message(request=request)

    def format_response(
        self,           # type: GetTotaledMessageProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        request,        # type: str
        response,       # type: Optional[TotaledMessage]
    ):
        # type: (...) -> None

        totalization_message = {
            "en": "",
            "es": "",
            "pt": "",
        }
        if response is not None:
            totalization_message["en"] = response.totalization_message.en
            totalization_message["es"] = response.totalization_message.es
            totalization_message["pt"] = response.totalization_message.pt

        totaled_message = {"totalizationMessage": totalization_message}
        data = json.dumps(totaled_message).encode("utf-8")

        message_bus.reply_message(
            message=message,
            reply_message=AckMessage(
                data_type=DataType.json,
                data=data,
            )
        )



from messagebus import (
    TokenCreator,
    TokenPriority,
)

TK_I18N_GETMSG = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group="7",
    number="2",
)
TK_I18N_GETLANGS = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group="7",
    number="4"
)
TK_I18N_GETTABLEJSON = TokenCreator.create_token(
    priority=TokenPriority.low,
    message_group="7",
    number="5",
)



"""Basic http server for tests to simulate PyPI or custom indexes"""

import http.server
import os
import threading
import time
import urllib.parse
import urllib.request


class IndexServer(http.server.HTTPServer):
    """Basic single-threaded http server simulating a package index

    You can use this server in unittest like this::
        s = IndexServer()
        s.start()
        index_url = s.base_url() + 'mytestindex'
        # do some test requests to the index
        # The index files should be located in setuptools/tests/indexes
        s.stop()
    """

    def __init__(
        self,
        server_address=('', 0),
        RequestHandlerClass=http.server.SimpleHTTPRequestHandler,
    ):
        http.server.HTTPServer.__init__(self, server_address, RequestHandlerClass)
        self._run = True

    def start(self):
        self.thread = threading.Thread(target=self.serve_forever)
        self.thread.start()

    def stop(self):
        "Stop the server"

        # Let the server finish the last request and wait for a new one.
        time.sleep(0.1)

        self.shutdown()
        self.thread.join()
        self.socket.close()

    def base_url(self):
        port = self.server_port
        return 'http://127.0.0.1:%s/setuptools/tests/indexes/' % port


class RequestRecorder(http.server.BaseHTTPRequestHandler):
    def do_GET(self):
        requests = vars(self.server).setdefault('requests', [])
        requests.append(self)
        self.send_response(200, 'OK')


class MockServer(http.server.HTTPServer, threading.Thread):
    """
    A simple HTTP Server that records the requests made to it.
    """

    def __init__(self, server_address=('', 0), RequestHandlerClass=RequestRecorder):
        http.server.HTTPServer.__init__(self, server_address, RequestHandlerClass)
        threading.Thread.__init__(self)
        self.daemon = True
        self.requests = []

    def run(self):
        self.serve_forever()

    @property
    def netloc(self):
        return 'localhost:%s' % self.server_port

    @property
    def url(self):
        return 'http://%s/' % self.netloc


def path_to_url(path, authority=None):
    """Convert a path to a file: URL."""
    path = os.path.normpath(os.path.abspath(path))
    base = 'file:'
    if authority is not None:
        base += '//' + authority
    return urllib.parse.urljoin(base, urllib.request.pathname2url(path))



"""sdist tests"""

from __future__ import annotations

import contextlib
import io
import itertools
import logging
import os
import shutil
import sys
import tempfile

import pytest

from setuptools.command.egg_info import FileList, egg_info, translate_pattern
from setuptools.dist import Distribution
from setuptools.tests.textwrap import DALS

from distutils import log
from distutils.errors import DistutilsTemplateError

IS_PYPY = '__pypy__' in sys.builtin_module_names


def make_local_path(s):
    """Converts '/' in a string to os.sep"""
    return s.replace('/', os.sep)


SETUP_ATTRS = {
    'name': 'app',
    'version': '0.0',
    'packages': ['app'],
}

SETUP_PY = (
    """\
from setuptools import setup

setup(**%r)
"""
    % SETUP_ATTRS
)


@contextlib.contextmanager
def quiet():
    old_stdout, old_stderr = sys.stdout, sys.stderr
    sys.stdout, sys.stderr = io.StringIO(), io.StringIO()
    try:
        yield
    finally:
        sys.stdout, sys.stderr = old_stdout, old_stderr


def touch(filename):
    open(filename, 'wb').close()


# The set of files always in the manifest, including all files in the
# .egg-info directory
default_files = frozenset(
    map(
        make_local_path,
        [
            'README.rst',
            'MANIFEST.in',
            'setup.py',
            'app.egg-info/PKG-INFO',
            'app.egg-info/SOURCES.txt',
            'app.egg-info/dependency_links.txt',
            'app.egg-info/top_level.txt',
            'app/__init__.py',
        ],
    )
)


translate_specs: list[tuple[str, list[str], list[str]]] = [
    ('foo', ['foo'], ['bar', 'foobar']),
    ('foo/bar', ['foo/bar'], ['foo/bar/baz', './foo/bar', 'foo']),
    # Glob matching
    ('*.txt', ['foo.txt', 'bar.txt'], ['foo/foo.txt']),
    ('dir/*.txt', ['dir/foo.txt', 'dir/bar.txt', 'dir/.txt'], ['notdir/foo.txt']),
    ('*/*.py', ['bin/start.py'], []),
    ('docs/page-?.txt', ['docs/page-9.txt'], ['docs/page-10.txt']),
    # Globstars change what they mean depending upon where they are
    (
        'foo/**/bar',
        ['foo/bing/bar', 'foo/bing/bang/bar', 'foo/bar'],
        ['foo/abar'],
    ),
    (
        'foo/**',
        ['foo/bar/bing.py', 'foo/x'],
        ['/foo/x'],
    ),
    (
        '**',
        ['x', 'abc/xyz', '@nything'],
        [],
    ),
    # Character classes
    (
        'pre[one]post',
        ['preopost', 'prenpost', 'preepost'],
        ['prepost', 'preonepost'],
    ),
    (
        'hello[!one]world',
        ['helloxworld', 'helloyworld'],
        ['hellooworld', 'helloworld', 'hellooneworld'],
    ),
    (
        '[]one].txt',
        ['o.txt', '].txt', 'e.txt'],
        ['one].txt'],
    ),
    (
        'foo[!]one]bar',
        ['fooybar'],
        ['foo]bar', 'fooobar', 'fooebar'],
    ),
]
"""
A spec of inputs for 'translate_pattern' and matches and mismatches
for that input.
"""

match_params = itertools.chain.from_iterable(
    zip(itertools.repeat(pattern), matches)
    for pattern, matches, mismatches in translate_specs
)


@pytest.fixture(params=match_params)
def pattern_match(request):
    return map(make_local_path, request.param)


mismatch_params = itertools.chain.from_iterable(
    zip(itertools.repeat(pattern), mismatches)
    for pattern, matches, mismatches in translate_specs
)


@pytest.fixture(params=mismatch_params)
def pattern_mismatch(request):
    return map(make_local_path, request.param)


def test_translated_pattern_match(pattern_match):
    pattern, target = pattern_match
    assert translate_pattern(pattern).match(target)


def test_translated_pattern_mismatch(pattern_mismatch):
    pattern, target = pattern_mismatch
    assert not translate_pattern(pattern).match(target)


class TempDirTestCase:
    def setup_method(self, method):
        self.temp_dir = tempfile.mkdtemp()
        self.old_cwd = os.getcwd()
        os.chdir(self.temp_dir)

    def teardown_method(self, method):
        os.chdir(self.old_cwd)
        shutil.rmtree(self.temp_dir)


class TestManifestTest(TempDirTestCase):
    def setup_method(self, method):
        super().setup_method(method)

        f = open(os.path.join(self.temp_dir, 'setup.py'), 'w', encoding="utf-8")
        f.write(SETUP_PY)
        f.close()
        """
        Create a file tree like:
        - LICENSE
        - README.rst
        - testing.rst
        - .hidden.rst
        - app/
            - __init__.py
            - a.txt
            - b.txt
            - c.rst
            - static/
                - app.js
                - app.js.map
                - app.css
                - app.css.map
        """

        for fname in ['README.rst', '.hidden.rst', 'testing.rst', 'LICENSE']:
            touch(os.path.join(self.temp_dir, fname))

        # Set up the rest of the test package
        test_pkg = os.path.join(self.temp_dir, 'app')
        os.mkdir(test_pkg)
        for fname in ['__init__.py', 'a.txt', 'b.txt', 'c.rst']:
            touch(os.path.join(test_pkg, fname))

        # Some compiled front-end assets to include
        static = os.path.join(test_pkg, 'static')
        os.mkdir(static)
        for fname in ['app.js', 'app.js.map', 'app.css', 'app.css.map']:
            touch(os.path.join(static, fname))

    def make_manifest(self, contents):
        """Write a MANIFEST.in."""
        manifest = os.path.join(self.temp_dir, 'MANIFEST.in')
        with open(manifest, 'w', encoding="utf-8") as f:
            f.write(DALS(contents))

    def get_files(self):
        """Run egg_info and get all the files to include, as a set"""
        dist = Distribution(SETUP_ATTRS)
        dist.script_name = 'setup.py'
        cmd = egg_info(dist)
        cmd.ensure_finalized()

        cmd.run()

        return set(cmd.filelist.files)

    def test_no_manifest(self):
        """Check a missing MANIFEST.in includes only the standard files."""
        assert (default_files - set(['MANIFEST.in'])) == self.get_files()

    def test_empty_files(self):
        """Check an empty MANIFEST.in includes only the standard files."""
        self.make_manifest("")
        assert default_files == self.get_files()

    def test_include(self):
        """Include extra rst files in the project root."""
        self.make_manifest("include *.rst")
        files = default_files | set(['testing.rst', '.hidden.rst'])
        assert files == self.get_files()

    def test_exclude(self):
        """Include everything in app/ except the text files"""
        ml = make_local_path
        self.make_manifest(
            """
            include app/*
            exclude app/*.txt
            """
        )
        files = default_files | set([ml('app/c.rst')])
        assert files == self.get_files()

    def test_include_multiple(self):
        """Include with multiple patterns."""
        ml = make_local_path
        self.make_manifest("include app/*.txt app/static/*")
        files = default_files | set([
            ml('app/a.txt'),
            ml('app/b.txt'),
            ml('app/static/app.js'),
            ml('app/static/app.js.map'),
            ml('app/static/app.css'),
            ml('app/static/app.css.map'),
        ])
        assert files == self.get_files()

    def test_graft(self):
        """Include the whole app/static/ directory."""
        ml = make_local_path
        self.make_manifest("graft app/static")
        files = default_files | set([
            ml('app/static/app.js'),
            ml('app/static/app.js.map'),
            ml('app/static/app.css'),
            ml('app/static/app.css.map'),
        ])
        assert files == self.get_files()

    def test_graft_glob_syntax(self):
        """Include the whole app/static/ directory."""
        ml = make_local_path
        self.make_manifest("graft */static")
        files = default_files | set([
            ml('app/static/app.js'),
            ml('app/static/app.js.map'),
            ml('app/static/app.css'),
            ml('app/static/app.css.map'),
        ])
        assert files == self.get_files()

    def test_graft_global_exclude(self):
        """Exclude all *.map files in the project."""
        ml = make_local_path
        self.make_manifest(
            """
            graft app/static
            global-exclude *.map
            """
        )
        files = default_files | set([ml('app/static/app.js'), ml('app/static/app.css')])
        assert files == self.get_files()

    def test_global_include(self):
        """Include all *.rst, *.js, and *.css files in the whole tree."""
        ml = make_local_path
        self.make_manifest(
            """
            global-include *.rst *.js *.css
            """
        )
        files = default_files | set([
            '.hidden.rst',
            'testing.rst',
            ml('app/c.rst'),
            ml('app/static/app.js'),
            ml('app/static/app.css'),
        ])
        assert files == self.get_files()

    def test_graft_prune(self):
        """Include all files in app/, except for the whole app/static/ dir."""
        ml = make_local_path
        self.make_manifest(
            """
            graft app
            prune app/static
            """
        )
        files = default_files | set([ml('app/a.txt'), ml('app/b.txt'), ml('app/c.rst')])
        assert files == self.get_files()


class TestFileListTest(TempDirTestCase):
    """
    A copy of the relevant bits of distutils/tests/test_filelist.py,
    to ensure setuptools' version of FileList keeps parity with distutils.
    """

    @pytest.fixture(autouse=os.getenv("SETUPTOOLS_USE_DISTUTILS") == "stdlib")
    def _compat_record_logs(self, monkeypatch, caplog):
        """Account for stdlib compatibility"""

        def _log(_logger, level, msg, args):
            exc = sys.exc_info()
            rec = logging.LogRecord("distutils", level, "", 0, msg, args, exc)
            caplog.records.append(rec)

        monkeypatch.setattr(log.Log, "_log", _log)

    def get_records(self, caplog, *levels):
        return [r for r in caplog.records if r.levelno in levels]

    def assertNoWarnings(self, caplog):
        assert self.get_records(caplog, log.WARN) == []
        caplog.clear()

    def assertWarnings(self, caplog):
        if IS_PYPY and not caplog.records:
            pytest.xfail("caplog checks may not work well in PyPy")
        else:
            assert len(self.get_records(caplog, log.WARN)) > 0
            caplog.clear()

    def make_files(self, files):
        for file in files:
            file = os.path.join(self.temp_dir, file)
            dirname, basename = os.path.split(file)
            os.makedirs(dirname, exist_ok=True)
            touch(file)

    def test_process_template_line(self):
        # testing  all MANIFEST.in template patterns
        file_list = FileList()
        ml = make_local_path

        # simulated file list
        self.make_files([
            'foo.tmp',
            'ok',
            'xo',
            'four.txt',
            'buildout.cfg',
            # filelist does not filter out VCS directories,
            # it's sdist that does
            ml('.hg/last-message.txt'),
            ml('global/one.txt'),
            ml('global/two.txt'),
            ml('global/files.x'),
            ml('global/here.tmp'),
            ml('f/o/f.oo'),
            ml('dir/graft-one'),
            ml('dir/dir2/graft2'),
            ml('dir3/ok'),
            ml('dir3/sub/ok.txt'),
        ])

        MANIFEST_IN = DALS(
            """\
        include ok
        include xo
        exclude xo
        include foo.tmp
        include buildout.cfg
        global-include *.x
        global-include *.txt
        global-exclude *.tmp
        recursive-include f *.oo
        recursive-exclude global *.x
        graft dir
        prune dir3
        """
        )

        for line in MANIFEST_IN.split('\n'):
            if not line:
                continue
            file_list.process_template_line(line)

        wanted = [
            'buildout.cfg',
            'four.txt',
            'ok',
            ml('.hg/last-message.txt'),
            ml('dir/graft-one'),
            ml('dir/dir2/graft2'),
            ml('f/o/f.oo'),
            ml('global/one.txt'),
            ml('global/two.txt'),
        ]

        file_list.sort()
        assert file_list.files == wanted

    def test_exclude_pattern(self):
        # return False if no match
        file_list = FileList()
        assert not file_list.exclude_pattern('*.py')

        # return True if files match
        file_list = FileList()
        file_list.files = ['a.py', 'b.py']
        assert file_list.exclude_pattern('*.py')

        # test excludes
        file_list = FileList()
        file_list.files = ['a.py', 'a.txt']
        file_list.exclude_pattern('*.py')
        file_list.sort()
        assert file_list.files == ['a.txt']

    def test_include_pattern(self):
        # return False if no match
        file_list = FileList()
        self.make_files([])
        assert not file_list.include_pattern('*.py')

        # return True if files match
        file_list = FileList()
        self.make_files(['a.py', 'b.txt'])
        assert file_list.include_pattern('*.py')

        # test * matches all files
        file_list = FileList()
        self.make_files(['a.py', 'b.txt'])
        file_list.include_pattern('*')
        file_list.sort()
        assert file_list.files == ['a.py', 'b.txt']

    def test_process_template_line_invalid(self):
        # invalid lines
        file_list = FileList()
        for action in (
            'include',
            'exclude',
            'global-include',
            'global-exclude',
            'recursive-include',
            'recursive-exclude',
            'graft',
            'prune',
            'blarg',
        ):
            try:
                file_list.process_template_line(action)
            except DistutilsTemplateError:
                pass
            except Exception:
                assert False, "Incorrect error thrown"
            else:
                assert False, "Should have thrown an error"

    def test_include(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # include
        file_list = FileList()
        self.make_files(['a.py', 'b.txt', ml('d/c.py')])

        file_list.process_template_line('include *.py')
        file_list.sort()
        assert file_list.files == ['a.py']
        self.assertNoWarnings(caplog)

        file_list.process_template_line('include *.rb')
        file_list.sort()
        assert file_list.files == ['a.py']
        self.assertWarnings(caplog)

    def test_exclude(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # exclude
        file_list = FileList()
        file_list.files = ['a.py', 'b.txt', ml('d/c.py')]

        file_list.process_template_line('exclude *.py')
        file_list.sort()
        assert file_list.files == ['b.txt', ml('d/c.py')]
        self.assertNoWarnings(caplog)

        file_list.process_template_line('exclude *.rb')
        file_list.sort()
        assert file_list.files == ['b.txt', ml('d/c.py')]
        self.assertWarnings(caplog)

    def test_global_include(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # global-include
        file_list = FileList()
        self.make_files(['a.py', 'b.txt', ml('d/c.py')])

        file_list.process_template_line('global-include *.py')
        file_list.sort()
        assert file_list.files == ['a.py', ml('d/c.py')]
        self.assertNoWarnings(caplog)

        file_list.process_template_line('global-include *.rb')
        file_list.sort()
        assert file_list.files == ['a.py', ml('d/c.py')]
        self.assertWarnings(caplog)

    def test_global_exclude(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # global-exclude
        file_list = FileList()
        file_list.files = ['a.py', 'b.txt', ml('d/c.py')]

        file_list.process_template_line('global-exclude *.py')
        file_list.sort()
        assert file_list.files == ['b.txt']
        self.assertNoWarnings(caplog)

        file_list.process_template_line('global-exclude *.rb')
        file_list.sort()
        assert file_list.files == ['b.txt']
        self.assertWarnings(caplog)

    def test_recursive_include(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # recursive-include
        file_list = FileList()
        self.make_files(['a.py', ml('d/b.py'), ml('d/c.txt'), ml('d/d/e.py')])

        file_list.process_template_line('recursive-include d *.py')
        file_list.sort()
        assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
        self.assertNoWarnings(caplog)

        file_list.process_template_line('recursive-include e *.py')
        file_list.sort()
        assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
        self.assertWarnings(caplog)

    def test_recursive_exclude(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # recursive-exclude
        file_list = FileList()
        file_list.files = ['a.py', ml('d/b.py'), ml('d/c.txt'), ml('d/d/e.py')]

        file_list.process_template_line('recursive-exclude d *.py')
        file_list.sort()
        assert file_list.files == ['a.py', ml('d/c.txt')]
        self.assertNoWarnings(caplog)

        file_list.process_template_line('recursive-exclude e *.py')
        file_list.sort()
        assert file_list.files == ['a.py', ml('d/c.txt')]
        self.assertWarnings(caplog)

    def test_graft(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # graft
        file_list = FileList()
        self.make_files(['a.py', ml('d/b.py'), ml('d/d/e.py'), ml('f/f.py')])

        file_list.process_template_line('graft d')
        file_list.sort()
        assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
        self.assertNoWarnings(caplog)

        file_list.process_template_line('graft e')
        file_list.sort()
        assert file_list.files == [ml('d/b.py'), ml('d/d/e.py')]
        self.assertWarnings(caplog)

    def test_prune(self, caplog):
        caplog.set_level(logging.DEBUG)
        ml = make_local_path
        # prune
        file_list = FileList()
        file_list.files = ['a.py', ml('d/b.py'), ml('d/d/e.py'), ml('f/f.py')]

        file_list.process_template_line('prune d')
        file_list.sort()
        assert file_list.files == ['a.py', ml('f/f.py')]
        self.assertNoWarnings(caplog)

        file_list.process_template_line('prune e')
        file_list.sort()
        assert file_list.files == ['a.py', ml('f/f.py')]
        self.assertWarnings(caplog)



from typing import (
    Optional,
)

from messagebus import Event
from ._AsyncTargetDto import AsyncTargetDto


class PublishEventRequest(object):
    def __init__(
        self,
        event: Event,
        target: Optional[str],
        async_target: Optional[AsyncTargetDto],
        external_code: Optional[str],
        fifo: bool
    ):
        self.event = event
        self.target = target
        self.async_target = async_target
        self.external_code = external_code
        self.fifo = fifo



from __future__ import annotations

import sys
import warnings
from typing import TYPE_CHECKING, Any, Literal

from typeguard._config import CollectionCheckStrategy, ForwardRefPolicy, global_config
from typeguard._exceptions import InstrumentationWarning
from typeguard._importhook import install_import_hook
from typeguard._utils import qualified_name, resolve_reference

if TYPE_CHECKING:
    from pytest import Config, Parser


def pytest_addoption(parser: Parser) -> None:
    def add_ini_option(
        opt_type: (
            Literal["string", "paths", "pathlist", "args", "linelist", "bool"] | None
        ),
    ) -> None:
        parser.addini(
            group.options[-1].names()[0][2:],
            group.options[-1].attrs()["help"],
            opt_type,
        )

    group = parser.getgroup("typeguard")
    group.addoption(
        "--typeguard-packages",
        action="store",
        help="comma separated name list of packages and modules to instrument for "
        "type checking, or :all: to instrument all modules loaded after typeguard",
    )
    add_ini_option("linelist")

    group.addoption(
        "--typeguard-debug-instrumentation",
        action="store_true",
        help="print all instrumented code to stderr",
    )
    add_ini_option("bool")

    group.addoption(
        "--typeguard-typecheck-fail-callback",
        action="store",
        help=(
            "a module:varname (e.g. typeguard:warn_on_error) reference to a function "
            "that is called (with the exception, and memo object as arguments) to "
            "handle a TypeCheckError"
        ),
    )
    add_ini_option("string")

    group.addoption(
        "--typeguard-forward-ref-policy",
        action="store",
        choices=list(ForwardRefPolicy.__members__),
        help=(
            "determines how to deal with unresolveable forward references in type "
            "annotations"
        ),
    )
    add_ini_option("string")

    group.addoption(
        "--typeguard-collection-check-strategy",
        action="store",
        choices=list(CollectionCheckStrategy.__members__),
        help="determines how thoroughly to check collections (list, dict, etc)",
    )
    add_ini_option("string")


def pytest_configure(config: Config) -> None:
    def getoption(name: str) -> Any:
        return config.getoption(name.replace("-", "_")) or config.getini(name)

    packages: list[str] | None = []
    if packages_option := config.getoption("typeguard_packages"):
        packages = [pkg.strip() for pkg in packages_option.split(",")]
    elif packages_ini := config.getini("typeguard-packages"):
        packages = packages_ini

    if packages:
        if packages == [":all:"]:
            packages = None
        else:
            already_imported_packages = sorted(
                package for package in packages if package in sys.modules
            )
            if already_imported_packages:
                warnings.warn(
                    f"typeguard cannot check these packages because they are already "
                    f"imported: {', '.join(already_imported_packages)}",
                    InstrumentationWarning,
                    stacklevel=1,
                )

        install_import_hook(packages=packages)

    debug_option = getoption("typeguard-debug-instrumentation")
    if debug_option:
        global_config.debug_instrumentation = True

    fail_callback_option = getoption("typeguard-typecheck-fail-callback")
    if fail_callback_option:
        callback = resolve_reference(fail_callback_option)
        if not callable(callback):
            raise TypeError(
                f"{fail_callback_option} ({qualified_name(callback.__class__)}) is not "
                f"a callable"
            )

        global_config.typecheck_fail_callback = callback

    forward_ref_policy_option = getoption("typeguard-forward-ref-policy")
    if forward_ref_policy_option:
        forward_ref_policy = ForwardRefPolicy.__members__[forward_ref_policy_option]
        global_config.forward_ref_policy = forward_ref_policy

    collection_check_strategy_option = getoption("typeguard-collection-check-strategy")
    if collection_check_strategy_option:
        collection_check_strategy = CollectionCheckStrategy.__members__[
            collection_check_strategy_option
        ]
        global_config.collection_check_strategy = collection_check_strategy



from application.model.configuration import Configurations


class LoyaltyIdentifierInteractor(object):
    def __init__(
        self,
        configs,    # type: Configurations
    ):
        # type: (...) -> None

        self.logger = configs.logger
        self._cache = {
            'LoyaltyIdentifierType': configs.id_type,
        }

    def get_key(
        self,
        key,    # type: str
    ):
        # type: (...) -> str

        return self._cache.get(key, None)



class OrderPicture(object):
    def __init__(
        self,
        store_id,      # type: str
        order_info,    # type: dict
    ):
        self.store_code = store_id
        self.order_info = order_info



# nrt.py - functions for handling Andorra NRT numbers
# coding: utf-8
#
# Copyright (C) 2019 Leandro Regueiro
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""NRT (Número de Registre Tributari, Andorra tax number).

The Número de Registre Tributari (NRT) is an identifier of legal and natural
entities for tax purposes.

This number consists of one letter indicating the type of entity, then 6
digits, followed by a check letter.

More information:

* https://www.oecd.org/tax/automatic-exchange/crs-implementation-and-assistance/tax-identification-numbers/Andorra-TIN.pdf

>>> validate('U-132950-X')
'U132950X'
>>> validate('A123B')
Traceback (most recent call last):
    ...
InvalidLength: ...
>>> validate('I 706193 G')
Traceback (most recent call last):
    ...
InvalidComponent: ...
>>> format('D059888N')
'D-059888-N'
"""  # noqa: E501

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation.

    This strips the number of any valid separators and removes surrounding
    whitespace.
    """
    return clean(number, ' -.').upper().strip()


def validate(number):
    """Check if the number is a valid Andorra NRT number.

    This checks the length, formatting and other constraints. It does not check
    for control letter.
    """
    number = compact(number)
    if len(number) != 8:
        raise InvalidLength()
    if not number[0].isalpha() or not number[-1].isalpha():
        raise InvalidFormat()
    if not isdigits(number[1:-1]):
        raise InvalidFormat()
    if number[0] not in 'ACDEFGLOPU':
        raise InvalidComponent()
    if number[0] == 'F' and number[1:-1] > '699999':
        raise InvalidComponent()
    if number[0] in 'AL' and not ('699999' < number[1:-1] < '800000'):
        raise InvalidComponent()
    return number


def is_valid(number):
    """Check if the number is a valid Andorra NRT number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    number = compact(number)
    return '-'.join([number[0], number[1:-1], number[-1]])



from logging import Logger

from typing import Dict

from production.box import NonComboFilterBox
from production.treebuilder import BoxConfiguration
from ._BoxBuilder import BoxBuilder


class NonComboFilterBoxBuilder(BoxBuilder):
    def __init__(
        self,                   # type: NonComboFilterBoxBuilder
        loggers,                # type: Dict[str, Logger]
    ):
        # type: (...) -> None

        super(NonComboFilterBoxBuilder, self).__init__(loggers=loggers)

    def build(
        self,       # type: NonComboFilterBoxBuilder
        config,     # type: BoxConfiguration
    ):
        # type: (...) -> NonComboFilterBox

        return NonComboFilterBox(
            name=config.name,
            sons=config.sons,
            logger=self.get_logger(config=config),
        )



import base64
import cPickle as Pickle
import os

import production.repository

from Queue import Queue, Empty
from datetime import (
    datetime,
    timedelta,
)
from logging import Logger
from threading import (
    Thread,
)

from dbrepository import SynchronizedDbApiTemplate
from production.model import (
    Item,
    ProductionOrder,
)
from typing import (
    List,
    Dict,
    Optional,
    Union,
)


class ProductionRepository(production.repository.ProductionRepository):

    def __init__(
        self,           # type: ProductionRepository
        loggers,        # type: Dict[unicode, Logger]
        connection,     # type: SynchronizedDbApiTemplate
    ):
        # type: (...) -> None

        self.loggers = loggers
        self.connection = connection

        self.order_queue = Queue()
        self.running = True

        self.processor_thread = Thread(target=self.consume_queue)
        self.processor_thread.daemon = True
        self.processor_thread.start()

    def consume_queue(
        self,   # type: ProductionRepository
    ):
        # type: (...) -> None

        while self.running:
            order_id = None
            try:
                order_to_save = self.order_queue.get(True, 5)
                order_id = order_to_save.order_id
                self._save_order_async(
                    order=order_to_save,
                )
            except Empty:
                continue
            except (Exception,):
                self.loggers["ERROR"].exception("Error saving order: {}".format(order_id))

    def stop(
        self,   # type: ProductionRepository
    ):
        # type: (...) -> None

        self.running = False

    def save_order(
        self,       # type: ProductionRepository
        order,      # type: ProductionOrder
    ):
        # type: (...) -> None

        try:
            self._remove_forbidden_tags(
                items=order.items,
                order_id=str(order.order_id),
            )
        except (Exception,):
            self.loggers["ERROR"].error("Error handling remove_forbidden_tags for order id {}".format(order.order_id))

        self.order_queue.put(order)

    def get_orders_by_orders_ids(
        self,           # type: ProductionRepository
        orders_ids,     # type: List,
    ):
        # type: (...) -> List[ProductionOrder]

        with self.connection as conn:
            orders = []

            chunks = [orders_ids[i:i + 100] for i in range(0, len(orders_ids), 100)]
            for chunk in chunks:
                def mapper(row):
                    order_data = base64.b64decode(row[0])
                    order = Pickle.loads(order_data)
                    orders.append(order)

                orders_list = [int(order_id) for order_id in chunk]
                placeholders = ",".join([
                    "?" for _ in chunk
                ])
                conn.execute(
                    sql=select_query.format(placeholders=placeholders),
                    params=tuple(orders_list),
                    mapper=mapper,
                )
            return orders

    def get_all_orders(
        self,   # type: ProductionRepository
    ):
        # type: (...) -> List[ProductionOrder]

        with self.connection as conn:
            orders_ids = []

            conn.execute(
                sql=select_orders_id_query,
                mapper=lambda row: orders_ids.append(row["OrderId"]),
            )

        return self.get_orders_by_orders_ids(orders_ids)

    def get_orders_to_purge(
        self,           # type: ProductionRepository
        limit_date,     # type: datetime
    ):
        # type: (...) -> List[ProductionOrder]

        with self.connection as conn:
            orders = []

            def mapper(row):
                order_id = int(row["OrderId"])
                orders.append(order_id)

            formatted_limit_date = limit_date.strftime("%Y-%m-%dT%H:%M:%S")
            conn.execute(
                sql=get_orders_to_purge_query,
                params=(formatted_limit_date,),
                mapper=mapper,
            )
            return orders

    def purge_orders(
        self,       # type: ProductionRepository
        orders,     # type: List
    ):
        # type: (...) -> None

        with self.connection as conn:
            orders_list = [int(order) for order in orders]
            placeholders = ",".join([
                "?" for _ in orders_list
            ])
            query = purge_orders_query
            query = query.format(placeholders=placeholders)
            conn.execute(
                sql=query,
                params=tuple(orders_list)
            )
            self._commit(
                connection=conn,
            )

    def save_pick_list_order(
        self,               # type: ProductionRepository
        order_id,           # type: int
        order_id_count,     # type: str
        jit,                # type: str
        course,             # type: str
        receipt,            # type: str
        status,             # type: bool
    ):
        # type: (...) -> None

        with self.connection as conn:
            date = datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")

            params = (
                order_id,
                order_id_count,
                jit,
                course,
                receipt,
                date,
                status,
            )
            conn.execute(
                sql=insert_pick_list_query,
                params=params,
            )
            self._commit(
                connection=conn,
            )

    def get_picklist_id(
        self,               # type: ProductionRepository
        order_id,           # type: int
        order_id_count,     # type: str
        jit,                # type: str
        course,             # type: str
    ):
        # type: (...) -> Optional[int]

        with self.connection as conn:
            params = (
                order_id,
                order_id_count,
                jit,
                course,
            )
            return conn.first(
                sql=select_get_pick_list_id_query,
                params=params,
            )

    def get_all_pick_list_orders(
        self,           # type: ProductionRepository
        orders_list,    # type: List[str]
    ):
        # type: (...) -> Dict

        with self.connection as conn:
            pick_list_orders = {}

            def mapper(row):
                jit = row["JIT"]
                course_name = row["Course"]

                if course_name != "":
                    course_name = " - {}".format(
                        course_name,
                    )

                key = jit + course_name
                pick_list_orders[key] = row["Id"]

            orders_list = [int(order) for order in orders_list]
            placeholders = ",".join([
                "?" for _ in orders_list
            ])
            query = select_pick_list_orders_query
            query = query.format(placeholders=placeholders)
            conn.execute(
                sql=query,
                params=tuple(orders_list),
                mapper=mapper,
            )
            return pick_list_orders

    def get_data_pick_list_order(
        self,                   # type: ProductionRepository
        pick_list_order_id,     # type: int
    ):
        # type: (...) -> str

        with self.connection as conn:
            return conn.first(
                sql=select_get_data_pick_list_order,
                params=(pick_list_order_id,),
            )

    def delete_outdated_pick_list_orders(
        self,           # type: ProductionRepository
        range_days,     # type: int
    ):
        # type: (...) -> None

        with self.connection as conn:
            self.loggers["INFO"].info("Start delete_outdated_pick_list_orders")

            result_datetime = datetime.utcnow() - timedelta(days=range_days)
            formatted_date = result_datetime.strftime("%Y-%m-%dT%H:%M:%S.%f")
            conn.execute(
                sql=purge_picklist_orders_query,
                params=(formatted_date,),
            )
            self._commit(
                connection=conn,
            )

            self.loggers["INFO"].info("End delete_outdated_pick_list_orders")

    def vacuum_database(
        self,   # type: ProductionRepository
    ):
        # type: (...) -> Dict[str, Union[Dict, int]]

        with self.connection as conn:
            db_info = {
                "db_info": self._get_db_file(
                    conn=conn,
                    db_name_to_find="production.db",
                )
            }
            db_file = db_info.get("db_info").get("db_file")
            db_info["old_file_size"] = self._get_file_size(file_name=db_file)

            db_name = db_info.get("db_info").get("db_name")
            query = "VACUUM {};".format(db_name)
            conn.execute(
                sql=query,
            )

            db_info["new_file_size"] = self._get_file_size(file_name=db_file)
            self.loggers["INFO"].info("Finished [vacuum_production_database]")

            return db_info

    def _remove_forbidden_tags(
        self,       # type: ProductionRepository
        items,      # type: List[Item]
        order_id,   # type: str
    ):
        # type: (...) -> None

        forbidden_tags = [
            "wait-prep-time"
        ]

        for item in items:
            for forbidden_tag in forbidden_tags:
                if item.has_tag(forbidden_tag):
                    item.remove_tag(forbidden_tag)
                    self.loggers["ERROR"].error("Denied TAG for Order ID {}: {}".format(order_id, forbidden_tag))

            if len(item.items) > 0:
                self._remove_forbidden_tags(
                    items=item.items,
                    order_id=order_id,
                )

    def _save_order_async(
        self,   # type: ProductionRepository
        order,  # type: ProductionOrder
    ):
        # type: (...) -> None

        with self.connection as conn:
            self.loggers["INFO"].info("Saving order: {} to Database".format(order.order_id))

            purge_order = 0
            order_data = base64.b64encode(Pickle.dumps(order, Pickle.HIGHEST_PROTOCOL))
            params = (
                order.order_id,
                order_data,
                order.created_at,
                purge_order,
            )
            conn.execute(
                sql=insert_query,
                params=params,
            )
            self._commit(
                connection=conn,
            )

    @staticmethod
    def _commit(
        connection,     # type: SynchronizedDbApiTemplate
    ):
        # type: (...) -> None

        try:
            connection.conn.commit()
        except (Exception,):
            connection.conn.rollback()
            raise

    @staticmethod
    def _get_db_file(
        conn,                # type: SynchronizedDbApiTemplate
        db_name_to_find,     # type: str
    ):
        # type: (...) -> Dict[str, str]

        def mapper(row):
            db_file = row[2]
            if db_file.endswith(db_name_to_find):
                return {
                    "db_name": row[1],
                    "db_file": db_file,
                }

        response = conn.first(
            sql="PRAGMA database_list;",
            mapper=mapper,
        )
        return response

    @staticmethod
    def _get_file_size(
        file_name,  # type: str
    ):
        # type: (...) -> int

        return os.path.getsize(file_name)


insert_query = """
INSERT OR REPLACE INTO ProductionOrders(
    OrderId,
    Object,
    CreatedAt,
    Purged
)
VALUES (?, ?, ?, ?)
"""

select_orders_id_query = """
SELECT OrderId FROM ProductionOrders WHERE PURGED = 0 ORDER BY CreatedAt DESC
"""

select_query = """
SELECT Object FROM ProductionOrders WHERE OrderId IN ({placeholders})
"""

get_orders_to_purge_query = """
SELECT OrderId FROM ProductionOrders WHERE CreatedAt < ? ORDER BY CreatedAt ASC LIMIT 750
"""

purge_orders_query = """
DELETE FROM ProductionOrders WHERE OrderId IN ({placeholders})
"""

insert_pick_list_query = """
INSERT OR REPLACE INTO PicklistOrders
(OrderId, OrderIdCount, JIT, Course, Data, CreatedAt, PrintOK)
VALUES
(?, ?, ?, ?, ?, ?, ?)
"""

select_get_pick_list_id_query = """
SELECT Id FROM PicklistOrders WHERE OrderId = ? AND OrderIdCount = ? AND JIT = ? AND Course = ?
"""

purge_picklist_orders_query = """
DELETE FROM PicklistOrders WHERE CreatedAt <= ?
"""

select_pick_list_orders_query = """
SELECT Id, JIT, Course FROM PicklistOrders WHERE OrderId in ({placeholders}) ORDER BY CreatedAt
"""

select_get_data_pick_list_order = """
SELECT Data FROM PicklistOrders WHERE Id == ?
"""



# flake8: noqa

import abc
import os
import sys
import pathlib
import warnings
from contextlib import suppress
from typing import Union


if sys.version_info >= (3, 10):
    from zipfile import Path as ZipPath  # type: ignore
else:
    from zipp import Path as ZipPath  # type: ignore


try:
    from typing import runtime_checkable  # type: ignore
except ImportError:

    def runtime_checkable(cls):  # type: ignore
        return cls


try:
    from typing import Protocol  # type: ignore
except ImportError:
    Protocol = abc.ABC  # type: ignore


class TraversableResourcesLoader:
    """
    Adapt loaders to provide TraversableResources and other
    compatibility.

    Used primarily for Python 3.9 and earlier where the native
    loaders do not yet implement TraversableResources.
    """

    def __init__(self, spec):
        self.spec = spec

    @property
    def path(self):
        return self.spec.origin

    def get_resource_reader(self, name):
        from . import readers, _adapters

        def _zip_reader(spec):
            with suppress(AttributeError):
                return readers.ZipReader(spec.loader, spec.name)

        def _namespace_reader(spec):
            with suppress(AttributeError, ValueError):
                return readers.NamespaceReader(spec.submodule_search_locations)

        def _available_reader(spec):
            with suppress(AttributeError):
                return spec.loader.get_resource_reader(spec.name)

        def _native_reader(spec):
            reader = _available_reader(spec)
            return reader if hasattr(reader, 'files') else None

        def _file_reader(spec):
            try:
                path = pathlib.Path(self.path)
            except TypeError:
                return None
            if path.exists():
                return readers.FileReader(self)

        return (
            # local ZipReader if a zip module
            _zip_reader(self.spec)
            or
            # local NamespaceReader if a namespace module
            _namespace_reader(self.spec)
            or
            # local FileReader
            _file_reader(self.spec)
            or
            # native reader if it supplies 'files'
            _native_reader(self.spec)
            or
            # fallback - adapt the spec ResourceReader to TraversableReader
            _adapters.CompatibilityFiles(self.spec)
        )


def wrap_spec(package):
    """
    Construct a package spec with traversable compatibility
    on the spec/loader/reader.

    Supersedes _adapters.wrap_spec to use TraversableResourcesLoader
    from above for older Python compatibility (<3.10).
    """
    from . import _adapters

    return _adapters.SpecLoaderAdapter(package.__spec__, TraversableResourcesLoader)


if sys.version_info >= (3, 9):
    StrPath = Union[str, os.PathLike[str]]
else:
    # PathLike is only subscriptable at runtime in 3.9+
    StrPath = Union[str, "os.PathLike[str]"]


def ensure_traversable(path):
    """
    Convert deprecated string arguments to traversables (pathlib.Path).
    """
    if not isinstance(path, str):
        return path

    warnings.warn(
        "String arguments are deprecated. Pass a Traversable instead.",
        DeprecationWarning,
        stacklevel=3,
    )

    return pathlib.Path(path)



"""distutils.command.build

Implements the Distutils 'build' command."""

import os
import sys
import sysconfig

from ..core import Command
from ..errors import DistutilsOptionError
from ..util import get_platform


def show_compilers():
    from ..ccompiler import show_compilers

    show_compilers()


class build(Command):
    description = "build everything needed to install"

    user_options = [
        ('build-base=', 'b', "base directory for build library"),
        ('build-purelib=', None, "build directory for platform-neutral distributions"),
        ('build-platlib=', None, "build directory for platform-specific distributions"),
        (
            'build-lib=',
            None,
            "build directory for all distribution (defaults to either build-purelib or build-platlib",
        ),
        ('build-scripts=', None, "build directory for scripts"),
        ('build-temp=', 't', "temporary build directory"),
        (
            'plat-name=',
            'p',
            f"platform name to build for, if supported [default: {get_platform()}]",
        ),
        ('compiler=', 'c', "specify the compiler type"),
        ('parallel=', 'j', "number of parallel build jobs"),
        ('debug', 'g', "compile extensions and libraries with debugging information"),
        ('force', 'f', "forcibly build everything (ignore file timestamps)"),
        ('executable=', 'e', "specify final destination interpreter path (build.py)"),
    ]

    boolean_options = ['debug', 'force']

    help_options = [
        ('help-compiler', None, "list available compilers", show_compilers),
    ]

    def initialize_options(self):
        self.build_base = 'build'
        # these are decided only after 'build_base' has its final value
        # (unless overridden by the user or client)
        self.build_purelib = None
        self.build_platlib = None
        self.build_lib = None
        self.build_temp = None
        self.build_scripts = None
        self.compiler = None
        self.plat_name = None
        self.debug = None
        self.force = False
        self.executable = None
        self.parallel = None

    def finalize_options(self):  # noqa: C901
        if self.plat_name is None:
            self.plat_name = get_platform()
        else:
            # plat-name only supported for windows (other platforms are
            # supported via ./configure flags, if at all).  Avoid misleading
            # other platforms.
            if os.name != 'nt':
                raise DistutilsOptionError(
                    "--plat-name only supported on Windows (try "
                    "using './configure --help' on your platform)"
                )

        plat_specifier = f".{self.plat_name}-{sys.implementation.cache_tag}"

        # Python 3.13+ with --disable-gil shouldn't share build directories
        if sysconfig.get_config_var('Py_GIL_DISABLED'):
            plat_specifier += 't'

        # Make it so Python 2.x and Python 2.x with --with-pydebug don't
        # share the same build directories. Doing so confuses the build
        # process for C modules
        if hasattr(sys, 'gettotalrefcount'):
            plat_specifier += '-pydebug'

        # 'build_purelib' and 'build_platlib' just default to 'lib' and
        # 'lib.<plat>' under the base build directory.  We only use one of
        # them for a given distribution, though --
        if self.build_purelib is None:
            self.build_purelib = os.path.join(self.build_base, 'lib')
        if self.build_platlib is None:
            self.build_platlib = os.path.join(self.build_base, 'lib' + plat_specifier)

        # 'build_lib' is the actual directory that we will use for this
        # particular module distribution -- if user didn't supply it, pick
        # one of 'build_purelib' or 'build_platlib'.
        if self.build_lib is None:
            if self.distribution.has_ext_modules():
                self.build_lib = self.build_platlib
            else:
                self.build_lib = self.build_purelib

        # 'build_temp' -- temporary directory for compiler turds,
        # "build/temp.<plat>"
        if self.build_temp is None:
            self.build_temp = os.path.join(self.build_base, 'temp' + plat_specifier)
        if self.build_scripts is None:
            self.build_scripts = os.path.join(
                self.build_base, 'scripts-%d.%d' % sys.version_info[:2]
            )

        if self.executable is None and sys.executable:
            self.executable = os.path.normpath(sys.executable)

        if isinstance(self.parallel, str):
            try:
                self.parallel = int(self.parallel)
            except ValueError:
                raise DistutilsOptionError("parallel should be an integer")

    def run(self):
        # Run all relevant sub-commands.  This will be some subset of:
        #  - build_py      - pure Python modules
        #  - build_clib    - standalone C libraries
        #  - build_ext     - Python extensions
        #  - build_scripts - (Python) scripts
        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)

    # -- Predicates for the sub-command list ---------------------------

    def has_pure_modules(self):
        return self.distribution.has_pure_modules()

    def has_c_libraries(self):
        return self.distribution.has_c_libraries()

    def has_ext_modules(self):
        return self.distribution.has_ext_modules()

    def has_scripts(self):
        return self.distribution.has_scripts()

    sub_commands = [
        ('build_py', has_pure_modules),
        ('build_clib', has_c_libraries),
        ('build_ext', has_ext_modules),
        ('build_scripts', has_scripts),
    ]



from typing import Optional


class PartCode(object):
    def __init__(
        self,
        id,                     # type: str
        context,                # type: str
        quantity_sold=None,     # type: Optional[int]
    ):
        # type: (...) -> None

        self.id = id
        self.context = context
        self.quantity_sold = quantity_sold



from random import getrandbits

from production.command import ProductionCommand


class PurgeOrdersCommand(ProductionCommand):
    def __init__(self, orders):
        super(PurgeOrdersCommand, self).__init__(None, None)
        self.orders = orders

    def get_hash_extra_value(self):
        return "PurgeOrdersCommand{}".format(getrandbits(40))



from future import standard_library
from future.utils import PY3

if PY3:
    from urllib.response import *
else:
    __future_module__ = True
    with standard_library.suspend_hooks():
        from urllib import (addbase,
                            addclosehook,
                            addinfo,
                            addinfourl)




class ErrorGettingPOSModel(Exception):

    def __init__(
        self,
        pos_id: int,
    ) -> None:
        super(ErrorGettingPOSModel, self).__init__(f"Could not retrieve state of pos id {pos_id}")



# util/__init__.py
# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
# <see AUTHORS file>
#
# This module is part of SQLAlchemy and is released under
# the MIT License: https://www.opensource.org/licenses/mit-license.php


from collections import defaultdict as defaultdict
from functools import partial as partial
from functools import update_wrapper as update_wrapper
from typing import TYPE_CHECKING

from . import preloaded as preloaded
from ._collections import coerce_generator_arg as coerce_generator_arg
from ._collections import coerce_to_immutabledict as coerce_to_immutabledict
from ._collections import column_dict as column_dict
from ._collections import column_set as column_set
from ._collections import EMPTY_DICT as EMPTY_DICT
from ._collections import EMPTY_SET as EMPTY_SET
from ._collections import FacadeDict as FacadeDict
from ._collections import flatten_iterator as flatten_iterator
from ._collections import has_dupes as has_dupes
from ._collections import has_intersection as has_intersection
from ._collections import IdentitySet as IdentitySet
from ._collections import immutabledict as immutabledict
from ._collections import LRUCache as LRUCache
from ._collections import merge_lists_w_ordering as merge_lists_w_ordering
from ._collections import NONE_SET as NONE_SET
from ._collections import ordered_column_set as ordered_column_set
from ._collections import OrderedDict as OrderedDict
from ._collections import OrderedIdentitySet as OrderedIdentitySet
from ._collections import OrderedProperties as OrderedProperties
from ._collections import OrderedSet as OrderedSet
from ._collections import PopulateDict as PopulateDict
from ._collections import Properties as Properties
from ._collections import ReadOnlyContainer as ReadOnlyContainer
from ._collections import ReadOnlyProperties as ReadOnlyProperties
from ._collections import ScopedRegistry as ScopedRegistry
from ._collections import sort_dictionary as sort_dictionary
from ._collections import ThreadLocalRegistry as ThreadLocalRegistry
from ._collections import to_column_set as to_column_set
from ._collections import to_list as to_list
from ._collections import to_set as to_set
from ._collections import unique_list as unique_list
from ._collections import UniqueAppender as UniqueAppender
from ._collections import update_copy as update_copy
from ._collections import WeakPopulateDict as WeakPopulateDict
from ._collections import WeakSequence as WeakSequence
from .compat import anext_ as anext_
from .compat import arm as arm
from .compat import athrow as athrow
from .compat import b as b
from .compat import b64decode as b64decode
from .compat import b64encode as b64encode
from .compat import cmp as cmp
from .compat import cpython as cpython
from .compat import dataclass_fields as dataclass_fields
from .compat import decode_backslashreplace as decode_backslashreplace
from .compat import dottedgetter as dottedgetter
from .compat import has_refcount_gc as has_refcount_gc
from .compat import inspect_getfullargspec as inspect_getfullargspec
from .compat import is64bit as is64bit
from .compat import local_dataclass_fields as local_dataclass_fields
from .compat import osx as osx
from .compat import py310 as py310
from .compat import py311 as py311
from .compat import py312 as py312
from .compat import py38 as py38
from .compat import py39 as py39
from .compat import pypy as pypy
from .compat import win32 as win32
from .concurrency import await_fallback as await_fallback
from .concurrency import await_only as await_only
from .concurrency import greenlet_spawn as greenlet_spawn
from .concurrency import is_exit_exception as is_exit_exception
from .deprecations import became_legacy_20 as became_legacy_20
from .deprecations import deprecated as deprecated
from .deprecations import deprecated_cls as deprecated_cls
from .deprecations import deprecated_params as deprecated_params
from .deprecations import moved_20 as moved_20
from .deprecations import warn_deprecated as warn_deprecated
from .langhelpers import add_parameter_text as add_parameter_text
from .langhelpers import as_interface as as_interface
from .langhelpers import asbool as asbool
from .langhelpers import asint as asint
from .langhelpers import assert_arg_type as assert_arg_type
from .langhelpers import attrsetter as attrsetter
from .langhelpers import bool_or_str as bool_or_str
from .langhelpers import chop_traceback as chop_traceback
from .langhelpers import class_hierarchy as class_hierarchy
from .langhelpers import classproperty as classproperty
from .langhelpers import clsname_as_plain_name as clsname_as_plain_name
from .langhelpers import coerce_kw_type as coerce_kw_type
from .langhelpers import constructor_copy as constructor_copy
from .langhelpers import constructor_key as constructor_key
from .langhelpers import counter as counter
from .langhelpers import create_proxy_methods as create_proxy_methods
from .langhelpers import decode_slice as decode_slice
from .langhelpers import decorator as decorator
from .langhelpers import dictlike_iteritems as dictlike_iteritems
from .langhelpers import duck_type_collection as duck_type_collection
from .langhelpers import ellipses_string as ellipses_string
from .langhelpers import EnsureKWArg as EnsureKWArg
from .langhelpers import FastIntFlag as FastIntFlag
from .langhelpers import format_argspec_init as format_argspec_init
from .langhelpers import format_argspec_plus as format_argspec_plus
from .langhelpers import generic_fn_descriptor as generic_fn_descriptor
from .langhelpers import generic_repr as generic_repr
from .langhelpers import get_annotations as get_annotations
from .langhelpers import get_callable_argspec as get_callable_argspec
from .langhelpers import get_cls_kwargs as get_cls_kwargs
from .langhelpers import get_func_kwargs as get_func_kwargs
from .langhelpers import getargspec_init as getargspec_init
from .langhelpers import has_compiled_ext as has_compiled_ext
from .langhelpers import HasMemoized as HasMemoized
from .langhelpers import (
    HasMemoized_ro_memoized_attribute as HasMemoized_ro_memoized_attribute,
)
from .langhelpers import hybridmethod as hybridmethod
from .langhelpers import hybridproperty as hybridproperty
from .langhelpers import inject_docstring_text as inject_docstring_text
from .langhelpers import iterate_attributes as iterate_attributes
from .langhelpers import map_bits as map_bits
from .langhelpers import md5_hex as md5_hex
from .langhelpers import memoized_instancemethod as memoized_instancemethod
from .langhelpers import memoized_property as memoized_property
from .langhelpers import MemoizedSlots as MemoizedSlots
from .langhelpers import method_is_overridden as method_is_overridden
from .langhelpers import methods_equivalent as methods_equivalent
from .langhelpers import (
    monkeypatch_proxied_specials as monkeypatch_proxied_specials,
)
from .langhelpers import non_memoized_property as non_memoized_property
from .langhelpers import NoneType as NoneType
from .langhelpers import only_once as only_once
from .langhelpers import (
    parse_user_argument_for_enum as parse_user_argument_for_enum,
)
from .langhelpers import PluginLoader as PluginLoader
from .langhelpers import portable_instancemethod as portable_instancemethod
from .langhelpers import quoted_token_parser as quoted_token_parser
from .langhelpers import ro_memoized_property as ro_memoized_property
from .langhelpers import ro_non_memoized_property as ro_non_memoized_property
from .langhelpers import rw_hybridproperty as rw_hybridproperty
from .langhelpers import safe_reraise as safe_reraise
from .langhelpers import set_creation_order as set_creation_order
from .langhelpers import string_or_unprintable as string_or_unprintable
from .langhelpers import symbol as symbol
from .langhelpers import TypingOnly as TypingOnly
from .langhelpers import (
    unbound_method_to_callable as unbound_method_to_callable,
)
from .langhelpers import walk_subclasses as walk_subclasses
from .langhelpers import warn as warn
from .langhelpers import warn_exception as warn_exception
from .langhelpers import warn_limited as warn_limited
from .langhelpers import wrap_callable as wrap_callable
from .preloaded import preload_module as preload_module
